import os
import glob
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt


'''

Python3 ~/PycharmProjects/MyBioTools/MyBioTools/boxplot_matrix_KEGG.py -in Kelp_dRep_fun_stats -out Kelp_dRep_fun_stats.txt -in_percent
Python3 ~/PycharmProjects/MyBioTools/MyBioTools/boxplot_matrix_KEGG.py -in Tara_NM_fun_stats -out Tara_NM_fun_stats.txt -in_percent

Rscript ~/PycharmProjects/MyBioTools/MyBioTools/COG_boxplot_last1row.R -i Kelp_dRep_fun_stats.txt -o Kelp_dRep_fun_stats.png
Rscript ~/PycharmProjects/MyBioTools/MyBioTools/COG_boxplot_last1row.R -i ko_B_percent_df_no_NA.csv -o ko_B_percent_df_no_NA.png

'''


def get_no_hidden_folder_list(wd):
    folder_list = []
    for each_folder in os.listdir(wd):
        if not each_folder.startswith('.'):
            folder_list.append(each_folder)
    return folder_list


def get_bin_to_ko_percent_dict(pwd_ko_stats, ko_id_all):

    bin_to_ko_percent = {}
    for each_ko in open(pwd_ko_stats):
        if not each_ko.startswith('Level'):
            each_ko_split = each_ko.strip().split('\t')
            each_ko_id = each_ko_split[1]
            each_ko_percent = float(each_ko_split[3])
            if each_ko_id not in ko_id_all:
                ko_id_all.append(each_ko_id)

            bin_to_ko_percent[each_ko_id] = each_ko_percent

    return bin_to_ko_percent


# input files
KEGG_annot_results_folder = '/Users/songweizhi/Desktop/SpongeMAGs_dRep_KEGG_wd'

# output files
ko_B_percent_csv_no_NA = '/Users/songweizhi/Desktop/ko_B_percent_df_no_NA.csv'
ko_C_percent_csv_no_NA = '/Users/songweizhi/Desktop/ko_C_percent_df_no_NA.csv'


# DB file
KO_description_B = '/Users/songweizhi/DB/KEGG_DB/KO_description_B.txt'
KO_description_C = '/Users/songweizhi/DB/KEGG_DB/KO_description_C.txt'


columns_to_drop_B = []
columns_to_drop_C = []


KO_description_B_dict = {}
for each_ko_B in open(KO_description_B):
    each_ko_B_split = each_ko_B.strip().split('\t')
    KO_description_B_dict[each_ko_B_split[0]] = each_ko_B_split[1]


KO_description_C_dict = {}
for each_ko_C in open(KO_description_C):
    each_ko_C_split = each_ko_C.strip().split('\t')
    KO_description_C_dict[each_ko_C_split[0]] = each_ko_C_split[1]


KEGG_wd_folder_list = get_no_hidden_folder_list(KEGG_annot_results_folder)

bin_id_list = []
ko_id_B_all = []
ko_id_C_all = []
ko_percent_all_bins_B = {}
ko_percent_all_bins_C = {}
for each_folder in KEGG_wd_folder_list:

    bin_id = each_folder.split('_KEGG_wd')[0]
    bin_id_list.append(bin_id)

    pwd_ko_stats_A = '%s/%s/%s_ko_stats_A.txt' % (KEGG_annot_results_folder, each_folder, bin_id)
    pwd_ko_stats_B = '%s/%s/%s_ko_stats_B.txt' % (KEGG_annot_results_folder, each_folder, bin_id)
    pwd_ko_stats_C = '%s/%s/%s_ko_stats_C.txt' % (KEGG_annot_results_folder, each_folder, bin_id)
    pwd_ko_stats_D = '%s/%s/%s_ko_stats_D.txt' % (KEGG_annot_results_folder, each_folder, bin_id)

    bin_to_ko_percent_B = get_bin_to_ko_percent_dict(pwd_ko_stats_B, ko_id_B_all)
    bin_to_ko_percent_C = get_bin_to_ko_percent_dict(pwd_ko_stats_C, ko_id_C_all)

    ko_percent_all_bins_B[bin_id] = bin_to_ko_percent_B
    ko_percent_all_bins_C[bin_id] = bin_to_ko_percent_C


bin_id_list = sorted(bin_id_list)
ko_id_B_all = sorted(ko_id_B_all)
ko_id_C_all = sorted(ko_id_C_all)


KO_description_B_dict['NA'] = 'NA'
KO_description_C_dict['NA'] = 'NA'

ko_id_B_all_description = ['B_%s_%s' % (i, KO_description_B_dict[i]) for i in ko_id_B_all]
ko_id_C_all_description = ['C_%s_%s' % (i, KO_description_C_dict[i]) for i in ko_id_C_all]


ko_percent_lol_B = []
ko_percent_lol_C = []
for each_bin in bin_id_list:
    current_bin_ko_B_percent_dict = ko_percent_all_bins_B[each_bin]
    current_bin_ko_C_percent_dict = ko_percent_all_bins_C[each_bin]
    percent_list_B = []
    for ko_id_B in ko_id_B_all:
        ko_B_percent = 0
        if ko_id_B in current_bin_ko_B_percent_dict:
            ko_B_percent = current_bin_ko_B_percent_dict[ko_id_B]
        percent_list_B.append(ko_B_percent)

    percent_list_C = []
    for ko_id_C in ko_id_C_all:
        ko_C_percent = 0
        if ko_id_C in current_bin_ko_C_percent_dict:
            ko_C_percent = current_bin_ko_C_percent_dict[ko_id_C]
        percent_list_C.append(ko_C_percent)

    ko_percent_lol_B.append(percent_list_B)
    ko_percent_lol_C.append(percent_list_C)

    # percent_list_str_B = [str(i) for i in percent_list_B]
    # percent_list_str_C = [str(i) for i in percent_list_C]
    # for_out_B = '%s,%s' % (each_bin, ','.join(percent_list_str_B))
    # for_out_C = '%s,%s' % (each_bin, ','.join(percent_list_str_C))
    # print(for_out_B)
    # print(for_out_C)


ko_B_percent_df = pd.DataFrame(np.array(ko_percent_lol_B), index=bin_id_list, columns=ko_id_B_all_description)
ko_C_percent_df = pd.DataFrame(np.array(ko_percent_lol_C), index=bin_id_list, columns=ko_id_C_all_description)


# ko_B_percent_df = pd.DataFrame(np.array(ko_percent_lol_B), index=bin_id_list, columns=ko_id_B_all)
# ko_C_percent_df = pd.DataFrame(np.array(ko_percent_lol_C), index=bin_id_list, columns=ko_id_C_all)


ko_B_percent_df_no_NA = ko_B_percent_df.loc[:, ko_B_percent_df.columns != 'B_NA_NA']
ko_C_percent_df_no_NA = ko_C_percent_df.loc[:, ko_C_percent_df.columns != 'C_NA_NA']


ko_B_percent_df_no_NA.to_csv(ko_B_percent_csv_no_NA, header=True)
ko_C_percent_df_no_NA.to_csv(ko_C_percent_csv_no_NA, header=True)


# print(ko_B_percent_df.shape)
# print(ko_B_percent_df.index)
# for each in ko_B_percent_df.index:
#     print(each)
#print(ko_B_percent_df)
# ko_B_percent_df_NA = ko_B_percent_df.loc[:, ko_B_percent_df.columns == 'NA']
# print(ko_B_percent_df_NA)
# print(ko_percent_df.index)
# print(ko_percent_df.columns)
# print(ko_percent_df.values)
# print(ko_C_percent_df.shape)
# print(ko_C_percent_df)
# ko_C_percent_df_no_NA.boxplot(grid=False, figsize=(100, 30))
# plt.savefig('/Users/songweizhi/Desktop/test_C.png', dpi=300)
# ko_B_percent_df_no_NA.boxplot(grid=False, figsize=(25, 15))
# HGT_value = ko_B_percent_df.loc['HGT_pcofg'].tolist()
# print(HGT_value)
# print(ko_id_B_all)
# print(len(HGT_value))
# print(len(ko_id_B_all))
# marker = [u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o',u'o']
# color_list = ['r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r','r']
# color_list = 'rrrrrrrrrrrrrrrrbbbrrrrrbbbrrrrrrrrrrggggrrrrrrrrrrr'
# print(len(color_list))
# print(HGT_value)
# plt.plot(HGT_value, linewidth=0, marker='o', markersize=7, markeredgewidth=0, color=color_list)
# plt.plot(HGT_value, 'ro')
# plt.scatter(HGT_value, ko_id_B_all)
# plt.xticks(rotation=270)
# plt.savefig('/Users/songweizhi/Desktop/test_B.png', dpi=300)
# #print(len(ko_B_percent_df_NA.values))



import argparse
from Bio import SeqIO


parser = argparse.ArgumentParser()

parser.add_argument('-in',
                    required=True,
                    help='input file')

args = vars(parser.parse_args())
input = args['in']
output_r1 = input.replace('.fasta', '_R1.fasta')
output_r2 = input.replace('.fasta', '_R2.fasta')


r1_handle = open(output_r1, 'w')
r2_handle = open(output_r2, 'w')

n = 0
for each in SeqIO.parse(input, 'fasta'):
    if n % 2 == 0:
        SeqIO.write(each, r1_handle, 'fasta')
    if n % 2 == 1:
        SeqIO.write(each, r2_handle, 'fasta')
    n += 1
r1_handle.close()
r2_handle.close()



# -*- coding: utf-8 -*-
import scrapy


class ToScrapeCSSSpider(scrapy.Spider):
    name = "toscrape-css"
    start_urls = [
        'http://quotes.toscrape.com/',
    ]

    def parse(self, response):
        for quote in response.css("div.quote"):
            yield {
                'text': quote.css("span.text::text").extract_first(),
                'author': quote.css("small.author::text").extract_first(),
                'tags': quote.css("div.tags > a.tag::text").extract()
            }

        next_page_url = response.css("li.next > a::attr(href)").extract_first()
        if next_page_url is not None:
            yield scrapy.Request(response.urljoin(next_page_url))




import os
import glob
import shutil


###################################### CONFIGURATION ######################################

nodes_number = 1
ppn_number = 1
memory = 10
walltime_needed = '05:59:00'
email = 'wythe1987@163.com'
#email = 'weizhi.song@student.unsw.edu.au'
#modules_needed = ['perl/5.20.1', 'infernal/1.1.1', 'blast+/2.2.31', 'hmmer/3.1b2', 'prodigal/2.6.3', 'tbl2asn/25.3', 'parallel/20160222', 'prokka/1.12']
modules_needed = []
#wd = '/Users/songweizhi/Desktop/meta-Antarctic'
wd = '/Users/songweizhi/Desktop/web'
outputs_folder = 'qsub_wget'

###########################################################################################

os.chdir(wd)

# create outputs folder
if not os.path.exists(outputs_folder):
    os.makedirs(outputs_folder)
else:
    shutil.rmtree(outputs_folder)
    os.makedirs(outputs_folder)


# Prepare header
line_1 = '#!/bin/bash\n'
line_2 = '#PBS -l nodes=' + str(nodes_number) + ':ppn=' + str(ppn_number) + '\n'
line_3 = '#PBS -l mem=' + str(memory) + 'gb\n'
line_4 = '#PBS -l walltime=' + walltime_needed + '\n'
line_5 = '#PBS -j oe\n'
line_6 = '#PBS -M ' + email + '\n'
line_7 = '#PBS -m ae\n\n'
header = line_1 + line_2 + line_3 + line_4 + line_5 + line_6 + line_7

# Prepare module lines
module_lines = ''
for module in modules_needed:
    module_lines += 'module load ' + module + '\n'

#print(header)
#print(module_lines)


################################################################
qsub_wd = '/srv/scratch/z5039045/Flow_cell'

for each_run in open('with_link_3.txt'):
    each_run = each_run.strip()
    output_file_handle = open('%s/qsub_wget_%s.sh' % (outputs_folder, each_run.strip().split('/')[-1]), 'w')
    #print(each_run)
    # if len(each_run) == 9:
    #     each_run_no_last_3_4 = each_run[:-3]
    # if len(each_run) == 10:
    #     each_run_no_last_3_4 = each_run[:-4]

    #print(each_run_no_last_3)
    #print(header)
    #print(module_lines)
    #print('cd /srv/scratch/z5039045/Antarctica')
    #print('wget ftp://ftp-trace.ncbi.nih.gov/sra/sra-instant/reads/ByRun/sra/SRR/%s/%s/%s.sra' % (each_run_no_last_3_4, each_run, each_run))
    output_file_handle.write(header)
    output_file_handle.write(module_lines)
    output_file_handle.write('cd %s/3\n' % qsub_wd)
    #output_file_handle.write('fq2fa %s_Q30.fastq %s_Q30.fasta\n' % (each_run, each_run))
    #output_file_handle.write('spades.py --meta --only-assembler -t %s -s combined_%s_reads.fasta -o combined_%s_k21-127 -k 21,33,55,77,99,127' % (ppn_number, each_run, each_run))
    #output_file_handle.write('/short/du5/wzs561/idba_512/bin/idba_ud --pre_correction --num_threads 14 --mink 20 --maxk 124 --step 20 --read combined_%s_reads.fasta --out combined_%s_k20-124' % (each_run, each_run))
    #output_file_handle.write('prokka --force --prefix %s --locustag %s --strain %s --outdir %s %s.fna\n' % (each_run, each_run, each_run, each_run, each_run))
    output_file_handle.write(each_run + ' --no-check-certificate\n')


    #output_file_handle.write('wget ftp://ftp-trace.ncbi.nih.gov/sra/sra-instant/reads/ByRun/sra/SRR/%s/%s/%s.sra\n' % (each_run_no_last_3_4, each_run, each_run))

    output_file_handle.close()




# spades.py --meta --only-assembler -t 6 -s combined_reads_m5_l100i250.fasta -o combined_reads_m5_l100i250_k21-125 -k 21,33,55,77,99,127

# /short/du5/wzs561/idba-master/bin/idba_ud --pre_correction --num_threads 14 --mink 20 --maxk 124 --step 20 --read GemSIM_100bp_m0.fasta --out GemSIM_100bp_m0_k20-124




import os

os.chdir('/Users/songweizhi/Desktop/test')

dataset = 'ns'

grouping_file = '%s_species_tree_grouping_modified.txt' % dataset
gtdbtk_op = '%s_gtdbtk.tsv' % dataset

out = '%s_out.txt' % dataset
out_sorted = '%s_out_sorted.txt' % dataset

bin_to_group_dict = {}
for each in open(grouping_file):
    each_split = each.strip().split(',')
    bin_to_group_dict[each_split[1]] = each_split[0]
print(bin_to_group_dict)


out_handle = open(out, 'w')
for each2 in open(gtdbtk_op):
    each2_split = each2.strip().split('\t')
    bin = each2_split[0]
    assign = each2_split[1]
    #print('%s\t%s\t%s' % (bin_to_group_dict[bin], bin, assign))
    out_handle.write('%s\t%s\t%s\n' % (bin_to_group_dict[bin], bin, assign))

out_handle.close()



os.system('cat %s | sort > %s' % (out, out_sorted))














#!/usr/bin/env python
from __future__ import division
import os
import re
import glob
import shutil
import argparse
import warnings
from datetime import datetime
from Bio import SeqIO, AlignIO, Align
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC, generic_dna
from Bio import SeqFeature as SF
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation
import multiprocessing as mp
from BioSAK.BioSAK_config import config_dict

get_SCG_tree_usage = '''
===================================== get SCG tree example commands =====================================

# for completed genome
BioSAK get_SCG_tree -i genomes -p NorthSea -x fasta -t 4 -nonmeta

# for metagenome-assembled genomes (MAGs) 
BioSAK get_SCG_tree -i genomes -p NorthSea -x fasta -t 4

Software dependencies:
module load hmmer/3.2.1
module load mafft/7.407
module load fasttree/2.1.10
module load R/3.5.3
module load blast+/2.9.0
module load prodigal/2.6.3

=========================================================================================================
'''


def report_and_log(message_for_report, log_file, keep_quiet):

    time_format = '[%Y-%m-%d %H:%M:%S]'
    with open(log_file, 'a') as log_handle:
        log_handle.write('%s %s\n' % ((datetime.now().strftime(time_format)), message_for_report))

    if keep_quiet is False:
        print('%s %s' % ((datetime.now().strftime(time_format)), message_for_report))


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def remove_empty_element(list_in):

    list_out = []
    for each_element in list_in:
        if each_element != '':
            list_out.append(each_element)

    return list_out


def get_program_path_dict(pwd_cfg_file):
    program_path_dict = {}
    for each in open(pwd_cfg_file):
        each_split = each.strip().split('=')
        program_name = each_split[0]
        program_path = each_split[1]

        # remove space if there are
        if program_name[-1] == ' ':
            program_name = program_name[:-1]
        if program_path[0] == ' ':
            program_path = program_path[1:]
        program_path_dict[program_name] = program_path

    return program_path_dict


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def export_aa_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.protein)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def remove_low_cov_and_consensus_columns(alignment_file_in, minimal_cov, min_consensus, alignment_file_out):

    def list_to_segments(list_in):

        segments_out = []
        current_element = None
        current_segment = [None, None]
        for each_element in list_in:

            # for the first ellment
            if current_element == None:
                current_element = each_element
                current_segment = [each_element, each_element]

            elif each_element == current_element + 1:
                current_segment[1] = each_element
                current_element = each_element

            elif each_element != current_element + 1:

                # add segment to list
                segments_out.append(current_segment)

                # resetting segment
                current_segment = [each_element, each_element]
                current_element = each_element

        # add segment to list
        segments_out.append(current_segment)

        return segments_out

    def remove_columns_from_msa(alignment_in, cols_to_remove):

        # get 0 based index of all wanted columns
        cols_to_remove_0_base = [(i - 1) for i in cols_to_remove]
        aln_cols_index_all = list(range(alignment_in.get_alignment_length()))
        aln_cols_index_wanted = []
        for i in aln_cols_index_all:
            if i not in cols_to_remove_0_base:
                aln_cols_index_wanted.append(i)

        # get wanted alignment segments
        wanted_segments = list_to_segments(aln_cols_index_wanted)

        # create an empty Alignment object
        alignment_new = Align.MultipleSeqAlignment([])
        for sequence in alignment_in:
            new_seq_object = Seq('')
            new_seq_record = SeqRecord(new_seq_object)
            new_seq_record.id = sequence.id
            new_seq_record.description = sequence.description
            alignment_new.append(new_seq_record)

        # add wanted columns to empty Alignment object
        for segment in wanted_segments:

            # for single column segment
            if segment[0] == segment[1]:
                segment_value = alignment_in[:, segment[0]]

                m = 0
                for each_seq in alignment_new:
                    each_seq.seq = Seq(str(each_seq.seq) + segment_value[m])
                    m += 1

            # for multiple columns segment
            else:
                segment_value = alignment_in[:, (segment[0]):(segment[1] + 1)]
                alignment_new += segment_value

        return alignment_new

    def remove_low_cov_columns(alignment_in, min_cov_cutoff):

        # get columns with low coverage
        sequence_number = len(alignment_in)
        total_col_num = alignment_in.get_alignment_length()
        low_cov_columns = []
        n = 0
        while n < total_col_num:
            current_column = alignment_in[:, n]
            dash_number = current_column.count('-')
            gap_percent = (dash_number / sequence_number) * 100

            if gap_percent > min_cov_cutoff:
                low_cov_columns.append(n + 1)

            n += 1

        # remove identified columns
        alignment_new = remove_columns_from_msa(alignment_in, low_cov_columns)

        return alignment_new

    def remove_low_consensus_columns(alignment_in, min_css_cutoff):

        # get columns with low coverage
        sequence_number = len(alignment_in)
        total_col_num = alignment_in.get_alignment_length()
        low_css_columns = []
        n = 0
        while n < total_col_num:
            current_column = alignment_in[:, n]

            # get all aa in current column
            aa_list = set()
            for aa in current_column:
                aa_list.add(aa)

            # get maximum aa percent
            most_abundant_aa_percent = 0
            for each_aa in aa_list:
                each_aa_percent = (current_column.count(each_aa) / sequence_number) * 100
                if each_aa_percent > most_abundant_aa_percent:
                    most_abundant_aa_percent = each_aa_percent

            # if maximum percent lower than provided cutoff, add current column to low consensus column list
            if most_abundant_aa_percent < min_css_cutoff:
                low_css_columns.append(n + 1)

            n += 1

        # remove identified columns
        alignment_new = remove_columns_from_msa(alignment_in, low_css_columns)

        return alignment_new

    # read in alignment
    alignment = AlignIO.read(alignment_file_in, "fasta")

    # remove_low_cov_columns
    alignment_cov = remove_low_cov_columns(alignment, minimal_cov)

    # remove_low_consensus_columns
    alignment_cov_css = remove_low_consensus_columns(alignment_cov, min_consensus)

    # write filtered alignment
    alignment_file_out_handle = open(alignment_file_out, 'w')
    for each_seq in alignment_cov_css:
        alignment_file_out_handle.write('>%s\n' % str(each_seq.id))
        alignment_file_out_handle.write('%s\n' % str(each_seq.seq))
    alignment_file_out_handle.close()


def prodigal_parser(seq_file, sco_file, prefix, output_folder):

    bin_ffn_file =     '%s.ffn' % prefix
    bin_faa_file =     '%s.faa' % prefix
    pwd_bin_ffn_file = '%s/%s'  % (output_folder, bin_ffn_file)
    pwd_bin_faa_file = '%s/%s'  % (output_folder, bin_faa_file)

    # get sequence id list
    id_to_sequence_dict = {}
    sequence_id_list = []
    for each_seq in SeqIO.parse(seq_file, 'fasta'):
        id_to_sequence_dict[each_seq.id] = str(each_seq.seq)
        sequence_id_list.append(each_seq.id)


    # get sequence to cds dict and sequence to transl_table dict
    current_seq_id = ''
    current_transl_table = ''
    current_seq_csd_list = []
    seq_to_cds_dict = {}
    seq_to_transl_table_dict = {}
    for each_cds in open(sco_file):
        if each_cds.startswith('# Sequence Data'):

            # add to dict
            if current_seq_id != '':
                seq_to_cds_dict[current_seq_id] = current_seq_csd_list
                seq_to_transl_table_dict[current_seq_id] = current_transl_table

            # reset value
            current_seq_id = each_cds.strip().split(';seqhdr=')[1][1:-1].split(' ')[0]
            current_transl_table = ''
            current_seq_csd_list = []

        elif each_cds.startswith('# Model Data'):
            current_transl_table = each_cds.strip().split(';')[-2].split('=')[-1]

        else:
            current_seq_csd_list.append('_'.join(each_cds.strip().split('_')[1:]))

    seq_to_cds_dict[current_seq_id] = current_seq_csd_list
    seq_to_transl_table_dict[current_seq_id] = current_transl_table


    bin_ffn_file_handle = open(pwd_bin_ffn_file, 'w')
    bin_faa_file_handle = open(pwd_bin_faa_file, 'w')
    gene_index = 1
    for seq_id in sequence_id_list:

        # create SeqRecord
        current_sequence = Seq(id_to_sequence_dict[seq_id])
        current_SeqRecord = SeqRecord(current_sequence, id=seq_id)
        current_SeqRecord.seq.alphabet = generic_dna
        transl_table = seq_to_transl_table_dict[seq_id]

        # add SeqFeature to SeqRecord
        for cds in seq_to_cds_dict[seq_id]:

            # define locus_tag id
            locus_tag_id = '%s_%s' % (prefix, "{:0>5}".format(gene_index))

            # define FeatureLocation
            cds_split = cds.split('_')
            cds_start = SF.ExactPosition(int(cds_split[0]))
            cds_end = SF.ExactPosition(int(cds_split[1]))
            cds_strand = cds_split[2]
            current_strand = None
            if cds_strand == '+':
                current_strand = 1
            if cds_strand == '-':
                current_strand = -1
            current_feature_location = FeatureLocation(cds_start, cds_end, strand=current_strand)

            # get nc sequence
            sequence_nc = ''
            if cds_strand == '+':
                sequence_nc = id_to_sequence_dict[seq_id][cds_start-1:cds_end]
            if cds_strand == '-':
                sequence_nc = str(Seq(id_to_sequence_dict[seq_id][cds_start-1:cds_end], generic_dna).reverse_complement())

            # translate to aa sequence
            sequence_aa = str(SeqRecord(Seq(sequence_nc)).seq.translate(table=transl_table))

            # remove * at the end
            sequence_aa = sequence_aa[:-1]

            # export nc and aa sequences
            export_dna_record(sequence_nc, locus_tag_id, '', bin_ffn_file_handle)
            export_aa_record(sequence_aa, locus_tag_id, '', bin_faa_file_handle)

            # Define feature type
            current_feature_type = 'CDS'

            # Define feature qualifiers
            current_qualifiers_dict = {}
            current_qualifiers_dict['locus_tag'] = locus_tag_id
            current_qualifiers_dict['transl_table'] = transl_table
            current_qualifiers_dict['translation'] = sequence_aa

            # Create a SeqFeature
            current_feature = SeqFeature(current_feature_location, type=current_feature_type, qualifiers=current_qualifiers_dict)

            # Append Feature to SeqRecord
            current_SeqRecord.features.append(current_feature)
            gene_index += 1

    bin_ffn_file_handle.close()
    bin_faa_file_handle.close()


def sep_combined_hmm(combined_hmm_file, hmm_profile_sep_folder, hmmfetch_exe, pwd_hmmstat_exe):

    # extract hmm profile id from phylo.hmm
    pwd_phylo_hmm_stat_txt = '%s/phylo.hmm.stat.txt' % hmm_profile_sep_folder
    hmmstat_cmd = '%s %s > %s' % (pwd_hmmstat_exe, combined_hmm_file, pwd_phylo_hmm_stat_txt)
    os.system(hmmstat_cmd)

    # get hmm profile id file
    hmm_id_list = []
    for each_profile in open(pwd_phylo_hmm_stat_txt):
        if not each_profile.startswith('#'):
            each_profile_split = each_profile.strip().split(' ')
            if each_profile_split != ['']:
                each_profile_split_no_space = []
                for each_element in each_profile_split:
                    if each_element != '':
                        each_profile_split_no_space.append(each_element)
                hmm_id_list.append(each_profile_split_no_space[2])

    for each_hmm_id in hmm_id_list:
        hmmfetch_cmd = '%s %s %s > %s/%s.hmm' % (hmmfetch_exe, combined_hmm_file, each_hmm_id, hmm_profile_sep_folder, each_hmm_id)
        os.system(hmmfetch_cmd)


def prodigal_worker(argument_list):

    input_genome = argument_list[0]
    input_genome_folder = argument_list[1]
    pwd_prodigal_exe = argument_list[2]
    nonmeta_mode = argument_list[3]
    pwd_prodigal_output_folder = argument_list[4]

    # prepare command (according to Prokka)
    input_genome_basename, input_genome_ext = os.path.splitext(input_genome)
    pwd_input_genome = '%s/%s' % (input_genome_folder, input_genome)
    pwd_output_sco = '%s/%s.sco' % (pwd_prodigal_output_folder, input_genome_basename)

    prodigal_cmd_meta = '%s -f sco -q -c -m -g 11 -p meta -i %s -o %s' % (
    pwd_prodigal_exe, pwd_input_genome, pwd_output_sco)
    prodigal_cmd_nonmeta = '%s -f sco -q -c -m -g 11 -i %s -o %s' % (
    pwd_prodigal_exe, pwd_input_genome, pwd_output_sco)

    if nonmeta_mode is True:
        prodigal_cmd = prodigal_cmd_nonmeta
    else:
        prodigal_cmd = prodigal_cmd_meta

    os.system(prodigal_cmd)

    # prepare ffn, faa and gbk files from prodigal output
    prodigal_parser(pwd_input_genome, pwd_output_sco, input_genome_basename, pwd_prodigal_output_folder)


def hmmsearch_worker(argument_list):

    faa_file_basename = argument_list[0]
    pwd_SCG_tree_wd = argument_list[1]
    pwd_hmmsearch_exe = argument_list[2]
    path_to_hmm = argument_list[3]
    pwd_faa_folder = argument_list[4]

    # run hmmsearch
    pwd_faa_file = '%s/%s.faa' % (pwd_faa_folder, faa_file_basename)
    os.system('%s -o /dev/null --domtblout %s/%s_hmmout.tbl %s %s' % (pwd_hmmsearch_exe, pwd_SCG_tree_wd, faa_file_basename, path_to_hmm, pwd_faa_file))

    # Reading the protein file in a dictionary
    proteinSequence = {}
    for seq_record in SeqIO.parse(pwd_faa_file, 'fasta'):
        proteinSequence[seq_record.id] = str(seq_record.seq)

    # Reading the hmmersearch table/extracting the protein part found beu hmmsearch out of the protein/Writing
    # each protein sequence that was extracted to a fasta file (one for each hmm in phylo.hmm
    hmm_id = ''
    hmm_name = ''
    hmm_pos1 = 0
    hmm_pos2 = 0
    hmm_score = 0
    pwd_hmmout_tbl = pwd_SCG_tree_wd + '/' + faa_file_basename + '_hmmout.tbl'
    with open(pwd_hmmout_tbl, 'r') as tbl:
        for line in tbl:
            if line[0] == "#": continue
            line = re.sub('\s+', ' ', line)
            splitLine = line.split(' ')

            if (hmm_id == ''):
                hmm_id = splitLine[4]
                hmm_name = splitLine[0]
                hmm_pos1 = int(splitLine[17]) - 1
                hmm_pos2 = int(splitLine[18])
                hmm_score = float(splitLine[13])
            elif (hmm_id == splitLine[4]):
                if (float(splitLine[13]) > hmm_score):
                    hmm_name = splitLine[0]
                    hmm_pos1 = int(splitLine[17]) - 1
                    hmm_pos2 = int(splitLine[18])
                    hmm_score = float(splitLine[13])
            else:
                file_out = open(pwd_SCG_tree_wd + '/' + hmm_id + '.fasta', 'a+')
                file_out.write('>' + faa_file_basename + '\n')
                if hmm_name != '':
                    seq = str(proteinSequence[hmm_name][hmm_pos1:hmm_pos2])
                    file_out.write(str(seq) + '\n')
                    file_out.close()
                hmm_id = splitLine[4]
                hmm_name = splitLine[0]
                hmm_pos1 = int(splitLine[17]) - 1
                hmm_pos2 = int(splitLine[18])
                hmm_score = float(splitLine[13])

        else:
            file_out = open(pwd_SCG_tree_wd + '/' + hmm_id + '.fasta', 'a+')
            file_out.write('>' + faa_file_basename + '\n')
            if hmm_name != '':
                seq = str(proteinSequence[hmm_name][hmm_pos1:hmm_pos2])
                file_out.write(str(seq) + '\n')
                file_out.close()


def convert_hmmalign_output(align_in, align_out):

    # read in alignment
    sequence_id_list = []
    sequence_seq_dict = {}
    for aligned_seq in open(align_in):
        aligned_seq_split = aligned_seq.strip().split(' ')
        aligned_seq_split = remove_empty_element(aligned_seq_split)

        if aligned_seq_split != []:
            aligned_seq_id = aligned_seq_split[0]
            aligned_seq_seq = aligned_seq_split[1]

            # add id to sequence id list
            if aligned_seq_id not in sequence_id_list:
                sequence_id_list.append(aligned_seq_id)

            # add seq to sequence seq dict
            if aligned_seq_id not in sequence_seq_dict:
                sequence_seq_dict[aligned_seq_id] = aligned_seq_seq
            else:
                sequence_seq_dict[aligned_seq_id] += aligned_seq_seq

    # write out
    align_out_handle = open(align_out, 'w')
    for sequence_id in sequence_id_list:
        sequence_seq = sequence_seq_dict[sequence_id]
        align_out_handle.write('>%s\n' % sequence_id)
        align_out_handle.write('%s\n' % sequence_seq)
    align_out_handle.close()


def hmmalign_worker(argument_list):
    fastaFile_basename = argument_list[0]
    pwd_SCG_tree_wd = argument_list[1]
    pwd_hmm_profile_folder = argument_list[2]
    pwd_hmmalign_exe = argument_list[3]

    pwd_hmm_file =    '%s/%s.hmm'               % (pwd_hmm_profile_folder, fastaFile_basename)
    pwd_seq_in =      '%s/%s.fasta'             % (pwd_SCG_tree_wd, fastaFile_basename)
    pwd_aln_out_tmp = '%s/%s_aligned_tmp.fasta' % (pwd_SCG_tree_wd, fastaFile_basename)
    pwd_aln_out =     '%s/%s_aligned.fasta'     % (pwd_SCG_tree_wd, fastaFile_basename)

    hmmalign_cmd = '%s --trim --outformat PSIBLAST %s %s > %s ; rm %s' % (pwd_hmmalign_exe, pwd_hmm_file, pwd_seq_in, pwd_aln_out_tmp, pwd_seq_in)
    os.system(hmmalign_cmd)

    # convert alignment format
    convert_hmmalign_output(pwd_aln_out_tmp, pwd_aln_out)

    # remove tmp alignment
    os.system('rm %s' % pwd_aln_out_tmp)


def get_SCG_tree(args, config_dict):

    # read in arguments
    input_genome_folder =   args['i']
    output_prefix =         args['p']
    file_extension =        args['x']
    num_threads =           args['t']
    nonmeta_mode =          args['nonmeta']

    # read in config file
    path_to_hmm =           config_dict['path_to_hmm']
    pwd_prodigal_exe =      config_dict['prodigal']
    pwd_hmmsearch_exe =     config_dict['hmmsearch']
    pwd_hmmfetch_exe =      config_dict['hmmfetch']
    pwd_hmmalign_exe =      config_dict['hmmalign']
    pwd_hmmstat_exe =       config_dict['hmmstat']
    pwd_fasttree_exe =      config_dict['fasttree']

    warnings.filterwarnings("ignore")
    minimal_cov_in_msa = 50
    min_consensus_in_msa = 25
    keep_quiet = False


    #################################################### check input ###################################################

    # check whether input genome exist
    input_genome_file_re = '%s/*.%s' % (input_genome_folder, file_extension)
    input_genome_file_name_list = [os.path.basename(file_name) for file_name in glob.glob(input_genome_file_re)]
    if input_genome_file_name_list == []:
        print('No input genome detected, program exited!')
        exit()


    ############################################# define file/folder names #############################################

    get_SCG_tree_wd =                    '%s_get_SCG_tree_wd'                   % (output_prefix)
    prodigal_output_folder =             '%s_1_prodigal_output'                 % (output_prefix)
    extract_and_align_SCG_wd =           '%s_2_extract_and_align_SCGs'          % (output_prefix)
    combined_alignment_file_tmp =        '%s_SCG_tree.aln'                      % (output_prefix)
    combined_alignment_file =            '%s_SCG_tree_cov%s_css%s.aln'          % (output_prefix, minimal_cov_in_msa, min_consensus_in_msa)
    newick_tree_file =                   '%s_SCG_tree.newick'                   % (output_prefix)
    hmm_profile_sep_folder =             '%s_hmm_profile_fetched'               % (output_prefix)

    pwd_log_file =                       '%s/%s_get_SCG_tree.log'               % (get_SCG_tree_wd, output_prefix)
    pwd_prodigal_output_folder =         '%s/%s'                                % (get_SCG_tree_wd, prodigal_output_folder)
    pwd_extract_and_align_SCG_wd =       '%s/%s'                                % (get_SCG_tree_wd, extract_and_align_SCG_wd)
    pwd_combined_alignment_file_tmp =    '%s/%s'                                % (get_SCG_tree_wd, combined_alignment_file_tmp)
    pwd_combined_alignment_file =        '%s/%s'                                % (get_SCG_tree_wd, combined_alignment_file)
    pwd_hmm_profile_sep_folder =         '%s/%s/%s'                             % (get_SCG_tree_wd, extract_and_align_SCG_wd, hmm_profile_sep_folder)
    pwd_newick_tree_file =               '%s/%s'                                % (get_SCG_tree_wd, newick_tree_file)


    # create wd
    force_create_folder(get_SCG_tree_wd)


    ######################################## run prodigal with multiprocessing #########################################

    # for report and log
    report_and_log(('Running Prodigal with %s cores for input genomes' % num_threads), pwd_log_file, keep_quiet)

    # create prodigal output folder
    force_create_folder(pwd_prodigal_output_folder)

    # get input genome list
    input_genome_file_re = '%s/*.%s' % (input_genome_folder, file_extension)
    input_genome_file_name_list = [os.path.basename(file_name) for file_name in glob.glob(input_genome_file_re)]

    # prepare arguments for prodigal_worker
    list_for_multiple_arguments_Prodigal = []
    for input_genome in input_genome_file_name_list:
        list_for_multiple_arguments_Prodigal.append([input_genome, input_genome_folder, pwd_prodigal_exe, nonmeta_mode, pwd_prodigal_output_folder])

    # run prodigal with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(prodigal_worker, list_for_multiple_arguments_Prodigal)
    pool.close()
    pool.join()


    ########################################### get species tree (hmmsearch) ###########################################

    # create wd
    force_create_folder(pwd_extract_and_align_SCG_wd)

    # for report and log
    report_and_log(('Running Hmmsearch with %s cores' % num_threads), pwd_log_file, keep_quiet)

    faa_file_re = '%s/*.faa' % pwd_prodigal_output_folder
    faa_file_list = [os.path.basename(file_name) for file_name in glob.glob(faa_file_re)]
    faa_file_list = sorted(faa_file_list)

    faa_file_basename_list = []
    for faa_file in faa_file_list:
        faa_file_basename, faa_file_extension = os.path.splitext(faa_file)
        faa_file_basename_list.append(faa_file_basename)

    # prepare arguments for hmmsearch_worker
    list_for_multiple_arguments_hmmsearch = []
    for faa_file_basename in faa_file_basename_list:
        list_for_multiple_arguments_hmmsearch.append([faa_file_basename, pwd_extract_and_align_SCG_wd, pwd_hmmsearch_exe, path_to_hmm, pwd_prodigal_output_folder])

    # run hmmsearch with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(hmmsearch_worker, list_for_multiple_arguments_hmmsearch)
    pool.close()
    pool.join()


    ############################################# get species tree (hmmalign) #############################################

    # for report and log
    report_and_log(('Running Hmmalign with %s cores' % num_threads), pwd_log_file, keep_quiet)

    # fetch combined hmm profiles
    force_create_folder(pwd_hmm_profile_sep_folder)
    sep_combined_hmm(path_to_hmm, pwd_hmm_profile_sep_folder, pwd_hmmfetch_exe, pwd_hmmstat_exe)

    # Call hmmalign to align all single fasta files with hmms
    files = os.listdir(pwd_extract_and_align_SCG_wd)
    fastaFiles = [i for i in files if i.endswith('.fasta')]

    # prepare arguments for hmmalign_worker
    list_for_multiple_arguments_hmmalign = []
    for fastaFile in fastaFiles:

        fastaFiles_basename = '.'.join(fastaFile.split('.')[:-1])
        list_for_multiple_arguments_hmmalign.append([fastaFiles_basename, pwd_extract_and_align_SCG_wd, pwd_hmm_profile_sep_folder, pwd_hmmalign_exe])

    # run hmmalign with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(hmmalign_worker, list_for_multiple_arguments_hmmalign)
    pool.close()
    pool.join()


    ################################### get species tree (Concatenating alignments) ####################################

    # for report and log
    report_and_log('Concatenating alignments', pwd_log_file, keep_quiet)

    # concatenating the single alignments
    concatAlignment = {}
    for element in faa_file_basename_list:
        concatAlignment[element] = ''

    # Reading all single alignment files and append them to the concatenated alignment
    files = os.listdir(pwd_extract_and_align_SCG_wd)
    fastaFiles = [i for i in files if i.endswith('.fasta')]
    for faa_file_basename in fastaFiles:
        fastaFile = pwd_extract_and_align_SCG_wd + '/' + faa_file_basename
        proteinSequence = {}
        alignmentLength = 0
        for seq_record_2 in SeqIO.parse(fastaFile, 'fasta'):
            proteinName = seq_record_2.id
            proteinSequence[proteinName] = str(seq_record_2.seq)
            alignmentLength = len(proteinSequence[proteinName])

        for element in faa_file_basename_list:
            if element in proteinSequence.keys():
                concatAlignment[element] += proteinSequence[element]
            else:
                concatAlignment[element] += '-' * alignmentLength

    # writing alignment to file
    file_out = open(pwd_combined_alignment_file_tmp, 'w')
    for element in faa_file_basename_list:
        file_out.write('>' + element + '\n' + concatAlignment[element] + '\n')
    file_out.close()

    # remove columns with low coverage and low consensus
    report_and_log(('Removing columns from concatenated alignment represented by <%s%s of genomes and with an amino acid consensus <%s%s' % (minimal_cov_in_msa, '%', min_consensus_in_msa, '%')), pwd_log_file, keep_quiet)
    remove_low_cov_and_consensus_columns(pwd_combined_alignment_file_tmp, minimal_cov_in_msa, min_consensus_in_msa, pwd_combined_alignment_file)


    ########################################### get species tree (fasttree) ############################################

    # for report and log
    report_and_log('Running FastTree', pwd_log_file, keep_quiet)

    # calling fasttree for tree calculation
    fasttree_cmd = '%s -quiet %s > %s' % (pwd_fasttree_exe, pwd_combined_alignment_file, pwd_newick_tree_file)
    os.system(fasttree_cmd)

    # for report and log
    report_and_log(('SCG tree exported to: %s' % newick_tree_file), pwd_log_file, keep_quiet)


    ############################################## remove temporary files ##############################################

    # remove temporary files
    report_and_log(('Deleting temporary files'), pwd_log_file, keep_quiet)

    os.system('rm -r %s' % pwd_combined_alignment_file_tmp)


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()

    # arguments for PI
    parser.add_argument('-i',             required=True,  help='input genome folder')
    parser.add_argument('-p',             required=True,  help='output prefix')
    parser.add_argument('-x',             required=False, default='fasta',     help='file extension')
    parser.add_argument('-nonmeta',       required=False, action="store_true", help='annotate Non-metagenome-assembled genomes (Non-MAGs)')
    parser.add_argument('-t',             required=False, type=int, default=1, help='number of threads, default: 1')

    args = vars(parser.parse_args())

    get_SCG_tree(args, config_dict)



import os
import glob
import argparse
from datetime import datetime
import multiprocessing as mp


get_genome_NCBI_parser_usage = '''
================================== get_genome_NCBI example commands ==================================

# Usage:
1. Download https://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/prokaryotes.txt
4. provide prokaryotes.txt to BioSAK with '-db'

# Example commands
BioSAK get_genome_NCBI -csv prokaryotes.txt -out gnm_dir -fna -name -t 8
BioSAK get_genome_NCBI -csv prokaryotes.txt -out gnm_dir -fna -faa -name -t 8
BioSAK get_genome_NCBI -csv prokaryotes.txt -id ids.txt -out gnm_dir -fna -gbff -name -t 8 

# id file format (the 19th col of the downloaded csv file, one id per line)
GCA_009840555.1
GCA_009840575.1

=====================================================================================================
'''


def genome_download_worker(argument_list):

    genome_record_split =      argument_list[0]
    downloaded_genome_folder = argument_list[1]
    get_fna =                  argument_list[2]
    get_faa =                  argument_list[3]
    get_gbff =                 argument_list[4]
    with_name =                argument_list[5]

    genome_name = genome_record_split[0][1:-1]
    genome_name_no_space = '_'.join(genome_name.split(' '))
    if ('(' in genome_name_no_space) and (')' in genome_name_no_space):
        genome_name_no_space_no_parenthesis_tmp = genome_name_no_space.replace('(', '_')
        genome_name_no_space = genome_name_no_space_no_parenthesis_tmp.replace(')', '')


    GenBank_FTP = genome_record_split[-2][1:-1]
    GenBank_FTP_id = GenBank_FTP.strip().split('/')[-1]
    assembly_id = genome_record_split[5]

    # prepare cmds
    fna_file =                  '%s_genomic.fna.gz'                     % (GenBank_FTP_id)
    faa_file =                  '%s_protein.faa.gz'                     % (GenBank_FTP_id)
    gbff_file =                 '%s_genomic.gbff.gz'                    % (GenBank_FTP_id)
    pwd_fna_file =              '%s/%s'                                 % (downloaded_genome_folder, fna_file)
    pwd_faa_file =              '%s/%s'                                 % (downloaded_genome_folder, faa_file)
    pwd_gbff_file =             '%s/%s'                                 % (downloaded_genome_folder, gbff_file)
    ftp_fna_file =              '%s/%s'                                 % (GenBank_FTP, fna_file)
    ftp_faa_file =              '%s/%s'                                 % (GenBank_FTP, faa_file)
    ftp_gbff_file =             '%s/%s'                                 % (GenBank_FTP, gbff_file)
    wget_fna_cmd =              'wget %s -P %s -q'                      % (ftp_fna_file, downloaded_genome_folder)
    wget_faa_cmd =              'wget %s -P %s -q'                      % (ftp_faa_file, downloaded_genome_folder)
    wget_gbff_cmd =             'wget %s -P %s -q'                      % (ftp_gbff_file, downloaded_genome_folder)
    gunzip_fna_cmd =            'gunzip %s'                             % (pwd_fna_file)
    gunzip_faa_cmd =            'gunzip %s'                             % (pwd_faa_file)
    gunzip_gbff_cmd =           'gunzip %s'                             % (pwd_gbff_file)
    rename_fna_cmd =            'mv %s/%s_genomic.fna %s/%s.fna'        % (downloaded_genome_folder, GenBank_FTP_id, downloaded_genome_folder, assembly_id)
    rename_faa_cmd =            'mv %s/%s_protein.faa %s/%s.faa'        % (downloaded_genome_folder, GenBank_FTP_id, downloaded_genome_folder, assembly_id)
    rename_gbff_cmd =           'mv %s/%s_genomic.gbff %s/%s.gbff'      % (downloaded_genome_folder, GenBank_FTP_id, downloaded_genome_folder, assembly_id)
    rename_fna_cmd_with_name =  'mv %s/%s_genomic.fna %s/%s_%s.fna'     % (downloaded_genome_folder, GenBank_FTP_id, downloaded_genome_folder, assembly_id, genome_name_no_space)
    rename_faa_cmd_with_name =  'mv %s/%s_protein.faa %s/%s_%s.faa'     % (downloaded_genome_folder, GenBank_FTP_id, downloaded_genome_folder, assembly_id, genome_name_no_space)
    rename_gbff_cmd_with_name = 'mv %s/%s_genomic.gbff %s/%s_%s.gbff'   % (downloaded_genome_folder, GenBank_FTP_id, downloaded_genome_folder, assembly_id, genome_name_no_space)

    # download, decompress and rename
    if get_fna is True:
        os.system(wget_fna_cmd)
        if os.path.isfile(pwd_fna_file) is True:
            os.system(gunzip_fna_cmd)
            if with_name is False:
                os.system(rename_fna_cmd)
            else:
                os.system(rename_fna_cmd_with_name)
        else:
            print('fna file not found in: %s/' % (GenBank_FTP))

    if get_faa is True:
        os.system(wget_faa_cmd)
        if os.path.isfile(pwd_faa_file) is True:
            os.system(gunzip_faa_cmd)
            if with_name is False:
                os.system(rename_faa_cmd)
            else:
                os.system(rename_faa_cmd_with_name)
        else:
            print('faa file not found in: %s/' % (GenBank_FTP))

    if get_gbff is True:
        os.system(wget_gbff_cmd)
        if os.path.isfile(pwd_gbff_file) is True:
            os.system(gunzip_gbff_cmd)
            if with_name is False:
                os.system(rename_gbff_cmd)
            else:
                os.system(rename_gbff_cmd_with_name)
        else:
            print('gbff file not found in: %s/' % (GenBank_FTP))


def download_GenBank_genome(args):

    csv_file =          args['csv']
    output_folder =     args['out']
    assembly_id_file =  args['id']
    get_fna =           args['fna']
    get_faa =           args['faa']
    get_gbff =          args['gbff']
    with_name =         args['name']
    num_threads =       args['t']

    time_format = '[%Y-%m-%d %H:%M:%S] '

    if (get_fna is False) and (get_faa is False) and (get_gbff is False):
        print(datetime.now().strftime(time_format) + 'Please specify at least one file type to download, program exited')
        exit()

    os.mkdir(output_folder)

    # report
    print(datetime.now().strftime(time_format) + 'Downloading genomes with %s cores' % (num_threads))

    provided_version_dict = {}
    assembly_id_set = set()
    assembly_id_set_no_version = set()
    if assembly_id_file is not None:
        for each_id in open(assembly_id_file):
            assembly_id_set.add(each_id.strip())
            assembly_id_set_no_version.add(each_id.strip().split('.')[0])
            provided_version_dict[each_id.strip().split('.')[0]] = each_id.strip()

    # download genome with multiprocessing
    different_version_found_dict = {}
    genomes_in_csv = set()
    genomes_in_csv_no_version = set()
    list_for_multiple_arguments_download_worker = []
    for genome_record in open(csv_file):
        if not genome_record.startswith('#Organism Name'):
            genome_record_split = genome_record.strip().split(',')
            assembly_id = genome_record_split[5][1:-1]
            assembly_id_no_version = genome_record_split[5][1:-1].split('.')[0]
            if assembly_id_file is None:
                list_for_multiple_arguments_download_worker.append([genome_record_split, output_folder, get_fna, get_faa, get_gbff, with_name])
                genomes_in_csv.add(assembly_id)
                genomes_in_csv_no_version.add(assembly_id.split('.')[0])
            else:
                if assembly_id in assembly_id_set:
                    list_for_multiple_arguments_download_worker.append([genome_record_split, output_folder, get_fna, get_faa, get_gbff, with_name])
                elif assembly_id_no_version in assembly_id_set_no_version:
                    list_for_multiple_arguments_download_worker.append([genome_record_split, output_folder, get_fna, get_faa, get_gbff, with_name])
                    different_version_found_dict[assembly_id_no_version] = assembly_id

    # run COG annotaion files with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(genome_download_worker, list_for_multiple_arguments_download_worker)
    pool.close()
    pool.join()

    # remove unzip broken files
    os.system('rm %s/*.gz' % output_folder)

    # get stat
    downloaded_file_re = '%s/*_*' % output_folder
    downloaded_file_list = [os.path.basename(file_name) for file_name in glob.glob(downloaded_file_re)]
    downloaded_file_list_no_version = [i.split('.')[0] for i in downloaded_file_list]
    failed_gnm_set = set()
    if assembly_id_file is None:
        for each_gnm in genomes_in_csv:
            each_gnm_no_version = each_gnm.split('.')[0]
            if each_gnm_no_version not in downloaded_file_list_no_version:
                failed_gnm_set.add(each_gnm)
    else:
        for each_provided_gnm in assembly_id_set:
            provided_gnm_no_version = each_provided_gnm.split('.')[0]
            if provided_gnm_no_version not in downloaded_file_list_no_version:
                failed_gnm_set.add(each_provided_gnm)

    if len(failed_gnm_set) > 0:
        print(datetime.now().strftime(time_format) + 'Failed genomes number: %s' % len(failed_gnm_set))
        failed_gnm_txt = '%s_failed.txt'% output_folder
        failed_gnm_txt_handle = open(failed_gnm_txt, 'w')
        for each_failed_gnm in failed_gnm_set:
            failed_gnm_txt_handle.write('%s\n' % each_failed_gnm)
        failed_gnm_txt_handle.close()
        print(datetime.now().strftime(time_format) + 'See details in: %s' % failed_gnm_txt)

    if len(different_version_found_dict) > 0:
        print(datetime.now().strftime(time_format) + 'Different version found for %s genomes' % len(different_version_found_dict))
        diff_version_gnm_txt = '%s_diff_version.txt'% output_folder
        diff_version_gnm_txt_handle = open(diff_version_gnm_txt, 'w')
        diff_version_gnm_txt_handle.write('Provided\tDownloaded\n')
        for each_diff_version_gnm in different_version_found_dict:
            diff_version_gnm_txt_handle.write('%s\t%s\n' % (provided_version_dict[each_diff_version_gnm], different_version_found_dict[each_diff_version_gnm]))
        diff_version_gnm_txt_handle.close()
        print(datetime.now().strftime(time_format) + 'See details in: %s' % diff_version_gnm_txt)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-db',         required=True,                       help='csv file downloaded from NCBI genome_browse')
    parser.add_argument('-out',         required=True,                       help='output folder')
    parser.add_argument('-id',          required=False, default=None,        help='assembly id (6th col in the csv file), one id per line')
    parser.add_argument('-fna',         required=False, action="store_true", help='download fna file')
    parser.add_argument('-faa',         required=False, action="store_true", help='download faa file')
    parser.add_argument('-gbff',        required=False, action="store_true", help='download gbff file')
    parser.add_argument('-name',        required=False, action="store_true", help='include genome name in the downloaded files')
    parser.add_argument('-t',           required=False, default=1, type=int, help='number of threads')
    args = vars(parser.parse_args())
    download_GenBank_genome(args)




import os


def get_hits_group(input_file_name, output_file_name):
    matches_2 = open(input_file_name)
    output_2_file = open(output_file_name, 'w')
    current_gene = ''
    group_member = []
    for match in matches_2:
        match_split = match.strip().split('\t')
        query_2 = match_split[0]
        target_2 = match_split[1]
        if current_gene == '':
            current_gene = query_2
            group_member.append(target_2)
        else:
            if query_2 == current_gene:
                if target_2 not in group_member:
                    group_member.append(target_2)
                else:
                    pass
            else:
                output_2_file.write('%s\t%s' % (current_gene, '\t'.join(group_member)) + '\n')
                current_gene = query_2
                group_member = []
                group_member.append(target_2)
    output_2_file.write('%s\t%s' % (current_gene, '\t'.join(group_member)) + '\n')
    output_2_file.close()


wd = '/Users/songweizhi/Desktop/get_core'
pwd_matches = '%s/%s' % (wd, 'all_vs_all_ffn.tab')
pwd_qualified_matches = '%s/%s' % (wd, 'qualified_matches.txt')
pwd_qualified_matches_sorted = '%s/%s' % (wd, 'qualified_matches_sorted.txt')
pwd_qualified_matches_sorted_one_line = '%s/%s' % (wd, 'qualified_matches_sorted_one_line.txt')
pwd_qualified_matches_sorted_one_line_reorded = '%s/%s' % (wd, 'qualified_matches_sorted_one_line_reorded.txt')
pwd_qualified_matches_sorted_one_line_reorded_sorted_uniq = '%s/%s' % (wd, 'qualified_matches_sorted_one_line_reorded_sorted.txt')
pwd_qualified_core_genes = '%s/%s' % (wd, 'qualified_core_genes.txt')

matches = open(pwd_matches)
qualified_matches = open(pwd_qualified_matches, 'w')

for match in matches:
    match_split = match.strip().split('\t')
    query = match_split[0]
    subject = match_split[1]
    identity = float(match_split[2])
    align_len = int(match_split[3])
    query_len = int(match_split[12])
    subject_len = int(match_split[13])
    query_split = query.split('_')
    subject_split = subject.split('_')
    query_bin_name = '_'.join(query_split[:-1])
    subject_bin_name = '_'.join(subject_split[:-1])
    coverage_q = float("{0:.2f}".format(float(align_len) * 100 / float(query_len)))
    coverage_s = float("{0:.2f}".format(float(align_len) * 100 / float(subject_len)))
    if (coverage_q >= 80) and (coverage_s >= 80) and (identity >= 70) and (align_len >= 500):
        qualified_matches.write('%s\t%s\n' % (query, subject))
qualified_matches.close()

os.system('cat %s | sort > %s' % (pwd_qualified_matches, pwd_qualified_matches_sorted))

get_hits_group(pwd_qualified_matches_sorted, pwd_qualified_matches_sorted_one_line)


pwd_qualified_matches_sorted_one_line_handle = open(pwd_qualified_matches_sorted_one_line)
pwd_qualified_matches_sorted_one_line_reorded_handle = open(pwd_qualified_matches_sorted_one_line_reorded, 'w')
for each in pwd_qualified_matches_sorted_one_line_handle:
    each_split = each.strip().split('\t')
    each_split_uniq = []
    for each in each_split:
        if each not in each_split_uniq:
            each_split_uniq.append(each)

    each_split_uniq_sorted = sorted(each_split_uniq)
    if len(each_split_uniq_sorted) > 1:
        pwd_qualified_matches_sorted_one_line_reorded_handle.write('\t'.join(each_split_uniq_sorted) + '\n')
pwd_qualified_matches_sorted_one_line_reorded_handle.close()

os.system('cat %s | sort | uniq > %s' % (pwd_qualified_matches_sorted_one_line_reorded, pwd_qualified_matches_sorted_one_line_reorded_sorted_uniq))

pwd_qualified_matches_sorted_one_line_reorded_sorted_uniq_handle = open(pwd_qualified_matches_sorted_one_line_reorded_sorted_uniq)
pwd_qualified_core_genes_handle = open(pwd_qualified_core_genes, 'w')

exported_group = []
for each in pwd_qualified_matches_sorted_one_line_reorded_sorted_uniq_handle:
    each_split = each.strip().split('\t')
    each_split_A = []
    each_split_B = []
    for each_2 in each_split:
        if each_2[0] == 'A':
            each_split_A.append(each_2)
        if each_2[0] == 'B':
            each_split_B.append(each_2)
    if (len(each_split_A) >= 3) and (len(each_split_B) >= 3) and (each_split[0] not in exported_group):
        pwd_qualified_core_genes_handle.write(each)
        exported_group.append(each_split[0])
pwd_qualified_core_genes_handle.close()


########################################################################################################################

transfers = open('%s/donor2recip.txt' % wd)

selected = []
chongfude = []
for each in transfers:
    each_split_n = each.strip().split('\t')
    print(each_split_n[0])
    if each_split_n[0] not in selected:
        selected.append(each_split_n[0])
    else:
        chongfude.append(each_split_n[0])

print(len(selected))
print(chongfude)

cores = open(pwd_qualified_core_genes)
for each_core in cores:
    each_core_split = each_core.strip().split('\t')
    existed = 0
    for each_gene in each_core_split:
        if each_gene in selected:
            existed = 1
    if existed == 0:
        print(each_core.strip())




#################################################### the simple one ####################################################

import argparse


parser = argparse.ArgumentParser()

parser.add_argument('-t', required=True, help='sequence file')
parser.add_argument('-i', required=False, type=int, help='mutation level')
parser.add_argument('-keep_cds', action="store_true", required=False, help='insert transfers only to non-coding regions')

args = vars(parser.parse_args())
input_seq_file_name = args['t']
mutation_level = args['i']
keep_cds = int(args['keep_cds'])


########################################### defined as required or optional ############################################

import argparse


parser = argparse.ArgumentParser(description='', add_help=False)
required = parser.add_argument_group('required arguments')
optional = parser.add_argument_group('optional arguments')

optional.add_argument('-h', action='help', help='Show this help message and exit')
required.add_argument('-snv_qc', dest='SNV_QC', nargs='?', required=True,  type=str, help='deepSNV QC file')
required.add_argument('-min_both', dest='MIN_BOTH', nargs='?', required=True, type=int, help='The minimum number of reads harboring SNV')

args = vars(parser.parse_args())
SNV_quality_file = args['SNV_QC']
min_var_reads_num = args['MIN_BOTH']


########################################### defined as required or optional ############################################

import argparse


parser = argparse.ArgumentParser(add_help=False)
subparsers = parser.add_subparsers(help="--", dest='subparser_name')


PrepIn_parser = subparsers.add_parser('PrepIn', formatter_class=CustomHelpFormatter, description='Runs tree, lineage_set, analyze, qa', epilog='Example: checkm lineage_wf ./bins ./output')
PrepIn_parser.add_argument('bin_folder', help="folder containing bins (fasta format)")
PrepIn_parser.add_argument('out_folder', help="folder to write output files")







########################################################################################################################
######################################################### file #########################################################
########################################################################################################################

#################### sep_path_basename_ext ####################

import os

def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


file_1 = 'input_sequence.fasta'
file_2 = '/Users/weizhisong/Softwares/artemis/act.txt'
file_path_1, file_basename_1, file_extension_1 = sep_path_basename_ext(file_1)
file_path_2, file_basename_2, file_extension_2 = sep_path_basename_ext(file_2)

print(file_path_1)          # .
print(file_basename_1)      # input_sequence
print(file_extension_1)     # .fasta
print(file_path_2)          # /Users/weizhisong/Softwares/artemis
print(file_basename_2)      # act
print(file_extension_2)     # .txt


######################## get file size ########################

text_file = '/Users/songweizhi/Desktop/a.txt'
text_file_size = os.stat(text_file).st_size


###############################################################

# Sometimes when we only care about certain parts of a tuple when unpacking, a dummy variable like _ is used as placeholder
___, filename = os.path.split('/home/luciano/.ssh/idrsa.pub')
print(filename)     # idrsa.pub


########################################################################################################################
######################################################## folder ########################################################
########################################################################################################################

##################### force_create_folder #####################

import os
import shutil

def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


################## get_no_hidden_folder_list ##################

import os
import glob

def get_no_hidden_folder_list(wd):
    folder_list = []
    for each_folder in os.listdir(wd):
        if not each_folder.startswith('.'):
            folder_list.append(each_folder)
    return folder_list


####################### get folder list #######################

wd = '/Users/songweizhi/Desktop/test_'
print(get_no_hidden_folder_list(wd))


######################## get file list ########################

file_re = '/Users/songweizhi/Desktop/test/*.fa'
file_list = [os.path.basename(file_name) for file_name in glob.glob(file_re)]
print(file_list)


#################### whether folder exist #####################

import os

# check whether folder exist
folder_1 = '/Users/songweizhi/Desktop/folder_1'
folder_2 = '/Users/songweizhi/Desktop/folder_2'
file_1 = '/Users/songweizhi/Desktop/folder_1/file.txt'
os.mkdir(folder_2)

print(os.path.isdir(folder_1))  # False
print(os.path.isdir(folder_2))  # True
print(os.path.isfile(file_1))   # False


#################### remove folders or files #####################

def rm_folder_file(target_re):

    target_list = glob.glob(target_re)

    for target in target_list:

        if os.path.isdir(target) is True:
            os.system('rm -r %s' % target)
            
        elif os.path.isfile(target) is True:
            os.system('rm %s' % target)





wd = '/Users/songweizhi/Desktop/new'
transfer_file = open('%s/donor2recip.txt' % wd)
core_genes = open('/Users/songweizhi/Desktop/get_core/qualified_core_genes.txt')


donor_list = []
for each in transfer_file:
    each_split = each.strip().split('\t')
    donor = each_split[0]
    if donor != 'donor_gene':
        donor_list.append(donor)


for each in core_genes:
    each_split = each.strip().split('\t')
    exist = 0
    for each1 in donor_list:
        if each1 in each_split:
            exist = 1
    if exist == 0:
        print(each.strip())


import os
import glob
import argparse
from Bio import SeqIO


##################################################### CONFIGURATION ####################################################

parser = argparse.ArgumentParser()

parser.add_argument('-f',
                    required=True,
                    help='path to bin folder')

parser.add_argument('-x',
                    required=True,
                    help='bin file extension')

args = vars(parser.parse_args())
bin_folder = args['f']
bin_ext = args['x']

if bin_folder[-1] == '/':
    bin_folder = bin_folder[:-1]

########################################################################################################################

output_file = 'binning_results.txt'

bin_re = '%s/*.%s' % (bin_folder, bin_ext)
bin_file_list = [os.path.basename(file_name) for file_name in glob.glob(bin_re)]


bin_folder_p = '/'.join(bin_folder.split('/')[:-1])
if bin_folder_p == '':
    bin_folder_p = '.'
pwd_output_file = '%s/%s' % (bin_folder_p, output_file)


output_file_handle = open(pwd_output_file, 'w')
for each_bin in bin_file_list:
    each_bin_basename, ext = os.path.splitext(each_bin)
    pwd_each_bin = '%s/%s' % (bin_folder, each_bin)
    for each_ctg in SeqIO.parse(pwd_each_bin, 'fasta'):
        for_write = '%s\t%s\n' % (each_ctg.id, each_bin_basename)
        output_file_handle.write(for_write)

output_file_handle.close()




import os
import argparse

get_genome_GTDB_parser_usage = '''
================================== get_genome_GTDB example commands ==================================

# Example commands
BioSAK get_genome_GTDB -id genome.txt -path genome_paths.tsv -dir release202/fastani -out op_gnm_folder

# -dir: the folder which hold genome_paths.tsv
# genome paths: in the fastani folder from GTDB auxillary_files
# example of genome id file (one id per line):
GCA_902608175.1
GCA_903901015.1
GCF_000007225.1

=====================================================================================================
'''


def get_genome_GTDB(args):

    gnm_id_txt =            args['id']
    gtdb_ref_gnm_path_txt = args['path']
    gtdb_ref_gnm_folder =   args['dir']
    op_gnm_folder =         args['out']

    os.mkdir(op_gnm_folder)

    gnms_not_found_txt = '%s_genomes_not_in_GTDB.txt' % op_gnm_folder

    # read in gtdb_ref_gnm_path_txt
    ref_gnm_to_path_dict = {}
    for each_ref_gnm in open(gtdb_ref_gnm_path_txt):
        ref_gnm_split = each_ref_gnm.strip().split(' ')
        ref_accession = ref_gnm_split[0].split('_genomic')[0]
        pwd_ref_gnm = '%s/%s%s' % (gtdb_ref_gnm_folder, ref_gnm_split[1], ref_gnm_split[0])
        ref_gnm_to_path_dict[ref_accession] = pwd_ref_gnm

    gnm_not_in_gtdb_set = set()
    for each_gnm in open(gnm_id_txt):
        gnm_id = each_gnm.strip()
        if gnm_id in ref_gnm_to_path_dict:
            pwd_gnm = ref_gnm_to_path_dict[gnm_id]
            cmd_cp = 'cp %s %s/' % (pwd_gnm, op_gnm_folder)
            cmd_gunzip = 'gunzip %s/%s_genomic.fna.gz' % (op_gnm_folder, gnm_id)
            cmd_rename = 'mv %s/%s_genomic.fna %s/%s.fna' % (op_gnm_folder, gnm_id, op_gnm_folder, gnm_id)
            os.system(cmd_cp)
            os.system(cmd_gunzip)
            os.system(cmd_rename)
        else:
            gnm_not_in_gtdb_set.add(gnm_id)

    if len(gnm_not_in_gtdb_set) > 0:
        print('Genomes not found in GTDB: %s, see details in %s' % (len(gnm_not_in_gtdb_set), gnms_not_found_txt))
        gnms_not_found_txt_handle = open(gnms_not_found_txt, 'w')
        for each_unfound_gnm in gnm_not_in_gtdb_set:
            gnms_not_found_txt_handle.write('%s\n' % each_unfound_gnm)
        gnms_not_found_txt_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-id',      required=True,  help='genome id')
    parser.add_argument('-path',    required=True,  help='genome_paths.tsv')
    parser.add_argument('-dir',     required=True,  help='the folder which hold genome_paths.tsv')
    parser.add_argument('-out',     required=True,  help='output folder')
    args = vars(parser.parse_args())
    get_genome_GTDB(args)

gnm_id_txt              = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/genomes_to_retrieve.txt'
gtdb_ref_gnm_path_txt   = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/GTDB_r202_genome_paths.tsv'
gtdb_ref_gnm_folder     = '/srv/scratch/z5039045/DB/GTDB_r202/release202/fastani'
op_gnm_folder           = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/op_gnm_folder'



from Bio import SeqIO


checkm_qa_6_txt = '/Users/songweizhi/Desktop/op_6.txt'
checkm_qa_9_fa  = '/Users/songweizhi/Desktop/op_9.fa'


mag_to_duplicated_marker_dict = {}
marker_name_to_identified_marker_dict = {}
for each_line in open(checkm_qa_6_txt):
    if not each_line.startswith('Bin Id	Marker Id	Gene Ids'):
        each_line_split = each_line.strip().split('\t')
        mag_id = each_line_split[0]
        marker_name = each_line_split[1]
        marker_name_with_mag_id = '%s___%s' % (mag_id, marker_name)
        identified_marker_list = each_line_split[2].split(',')

        if mag_id not in mag_to_duplicated_marker_dict:
            mag_to_duplicated_marker_dict[mag_id] = {marker_name}
        else:
            mag_to_duplicated_marker_dict[mag_id].add(marker_name)

        marker_name_to_identified_marker_dict[marker_name_with_mag_id] = identified_marker_list


print(mag_to_duplicated_marker_dict)
print(marker_name_to_identified_marker_dict)


for each_seq in SeqIO.parse(checkm_qa_9_fa, 'fasta'):
    seq_des = each_seq.description
    seq_des_split = seq_des.split(' ')
    print(seq_des_split)

    mag_id = seq_des_split[0]
    ctg_id = seq_des_split[1]









import random

def get_common_stop_sequence():
    positive_strand_stop_codon = ['TAA', 'TAG', 'TGA']
    negative_strand_stop_codon = ['TTA', 'CTA', 'TCA']
    nc_bases = ['A', 'T', 'G', 'C']
    random_pos = random.sample(positive_strand_stop_codon, 3)
    random_neg = random.sample(negative_strand_stop_codon, 3)
    random_nc = random.sample(nc_bases, 4)
    common_stop_sequence = '%s%s%s%s%s%s%s%s%s%s' % (random_pos[0],
                                                     random_nc[0],
                                                     random_pos[1],
                                                     random_nc[1],
                                                     random_pos[2],
                                                     random_neg[0],
                                                     random_nc[2],
                                                     random_neg[1],
                                                     random_nc[3],
                                                     random_neg[2])
    return common_stop_sequence



a = get_common_stop_sequence()
print(a)



import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


def bar_plotter(label_list, num_list, output_plot, y_axis_label):

    x_range = range(len(label_list))
    plt.bar(x_range, num_list, tick_label=label_list, align='center', alpha=0.2, linewidth=0)
    plt.xticks(x_range, label_list, rotation=270, fontsize=10, horizontalalignment='center')
    plt.title('')
    plt.ylabel(y_axis_label)
    plt.tight_layout()
    plt.savefig(output_plot, dpi=300)
    plt.close()




import re
from urllib.request import urlopen


# if has Chinese, apply decode()
html = urlopen("https://morvanzhou.github.io/static/scraping/basic-structure.html").read().decode('utf-8')


# get Page title
Page_title = re.findall(r"<title>(.+?)</title>", html)[0]
print('\n------Page_title------')
print(Page_title)


# get Page paragraph
Page_paragraph = re.findall(r"<p>(.*?)</p>", html, flags=re.DOTALL)[0]   # re.DOTALL if multi line
print('\n------Page_paragraph------')
print(Page_paragraph)


# select links
Page_links = re.findall(r'href="(.*?)"', html)
print('\n------Page_links------')
print(Page_links)




import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns


MAG_df = pd.read_csv('Grouped_boxplot_with_extra_values.csv')

box_order =   ['Kelp',   'Planktonic']
color_order = ['orange', 'lightblue']

sns.boxplot(data=MAG_df, x="Phylum", y="Frequency",
            hue="Lifestyle", hue_order=box_order, palette=color_order)

sns.despine(offset=10, trim=True)

# add dots
plt.plot(0 - 0.2, 0.25, alpha=1, marker='v', markersize=10, markeredgewidth=0, color='deepskyblue')
plt.plot(0 + 0.2, 0.52, alpha=1, marker='^', markersize=10, markeredgewidth=0, color='coral')
plt.plot(1 - 0.2, 0.55, alpha=1, marker='^', markersize=10, markeredgewidth=0, color='coral')
plt.plot(1 + 0.2, 0.275, alpha=1, marker='v', markersize=10, markeredgewidth=0, color='deepskyblue')

plt.tight_layout()
plt.savefig('Grouped_boxplot_with_extra_values.png', bbox_inches='tight', dpi=300)
plt.close()


# # generate pandas dataframe and export to file
# MAG_1 = ['MAG_1', 0.38, 'Kelp', 'Proteobacteria']
# MAG_2 = ['MAG_2', 0.5, 'Planktonic', 'Proteobacteria']
# MAG_3 = ['MAG_3', 0.6, 'Kelp', 'Actinobacteriota']
# MAG_4 = ['MAG_4', 0.5, 'Kelp', 'Proteobacteria']
# MAG_5 = ['MAG_5', 0.3, 'Planktonic', 'Actinobacteriota']
# MAG_6 = ['MAG_6', 0.2, 'Kelp', 'Proteobacteria']
# MAG_7 = ['MAG_7', 0.3, 'Planktonic', 'Proteobacteria']
# MAG_9 = ['MAG_9', 0.38, 'Planktonic', 'Actinobacteriota']
# MAG_10 = ['MAG_10', 0.2, 'Kelp', 'Actinobacteriota']

# MAG_data = [MAG_1, MAG_2, MAG_3, MAG_4, MAG_5, MAG_6, MAG_7, MAG_9, MAG_10]
# MAG_df = pd.DataFrame(MAG_data, columns=['MAG', 'Frequency', 'Lifestyle', 'Phylum'])

# MAG_df.to_csv('/Users/songweizhi/Desktop/Grouped_boxplot_with_extra_values.csv', index=False)




import sys
total = 100000000 + 1
point = total/100 # 100 percent
increment = total/50 # the number of '='
for i in range(total):
    if i % (5 * point) == 0:
        sys.stdout.write("\r[" + "=" * (i/increment) + "_" * ((total - i)/increment) + "]" + str(i/point) + "%")
        sys.stdout.flush()


from Bio import SeqIO
from Bio.Graphics import GenomeDiagram
from Bio.Graphics.GenomeDiagram import test_GenomeDiagram, Diagram
from reportlab.lib import colors
from reportlab.lib.units import cm
# test_GenomeDiagram.py (https://github.com/biopython/biopython/blob/master/Tests/test_GenomeDiagram.py)
# download test_GenomeDiagram.py and copy it to folder "Bio/Graphics/GenomeDiagram/".
# /Library/Frameworks/Python.framework/Versions/3.4/lib/python3.4/site-packages/Bio/Graphics/GenomeDiagram/

contig = SeqIO.read('/Users/songweizhi/Desktop/test.gbk', "genbank")







diagram = GenomeDiagram.Diagram()

# add a graph track...
gc_content_track = diagram.new_track(1,
                                     greytrack = True,
                                     name = contig.name + " GC content",
                                     greytrack_labels = True,
                                     greytrack_font = 'Helvetica',
                                     greytrack_fontsize = 12,
                                     height = 0.35,
                                     start = 0,
                                     end = len(contig),
                                     scale = True,
                                     scale_fontsize = 12,
                                     scale_ticks = 0)

gene_content_track = diagram.new_track(1,
                                       greytrack = True,
                                       name = contig.name,
                                       greytrack_labels = True,
                                       greytrack_font = 'Helvetica',
                                       greytrack_fontsize = 12,
                                       height = 0.35,
                                       start = 0,
                                       end = len(contig),
                                       scale = True,
                                       scale_fontsize = 6,
                                       scale_ticks = 1,
                                       scale_smalltick_interval = 10000,
                                       scale_largetick_interval = 10000)


gc_content_set = gc_content_track.new_set(type = "graph")
feature_set = gene_content_track.new_set(type = 'feature')

step = len(contig)//500
gc_content_set.new_graph(test_GenomeDiagram.apply_to_window(contig.seq,
                                                            step,
                                                            test_GenomeDiagram.calc_gc_content, step),
                         'GC content',
                         style= 'bar',
                         color=colors.lightgreen,
                         altcolor=colors.darkseagreen)


# add features to feature set
for feature in contig.features:
    if feature.type == "CDS":
        feature_set.add_feature(feature,
                                color = colors.lightblue,
                                label = True,
                                sigil = 'ARROW',
                                arrowshaft_height = 0.7,
                                arrowhead_length = 0.4,
                                label_size = 10,
                                label_position = "middle")


# Finally draw it in both formats,
diagram.draw(format = 'linear',
             orientation = 'landscape',
             tracklines = 0,
             pagesize=(50 * cm, 20 * cm),
             fragments = 1)

diagram.write('/Users/songweizhi/Desktop/test_1.eps', 'EPS')



from PIL import Image, ImageDraw, ImageFont

background = Image.new('RGB', color = (255, 255, 255), size = (2000, 1000))
add_text = ImageDraw.Draw(background)

# Andale Mono
font = ImageFont.truetype("/Library/Fonts/Arial Bold.ttf", 18)
hgts = ['L12-->B2', 'C1-->n22', 'C1-->D5', 'L6-->n11', 'L6-->G24', 'G8-->G7', 'G5-->G8', 'G5-->G26', 'G21-->G23', 'n19-->G21', 'G16-->G25', 'L12-->B2', 'C1-->n22', 'C1-->D5', 'L6-->n11', 'L6-->G24', 'G8-->G7', 'G5-->G8', 'G5-->G26', 'G21-->G23', 'n19-->G21', 'G16-->G25']
x_start = 30
y_start = 80

for hgt in hgts:

    if x_start < 900:
        if hgt == 'C1-->D5':
            add_text.text((x_start, y_start), hgt, (225, 0, 0), font = font)
            x_start += 120
        else:
            add_text.text((x_start, y_start), hgt, (0, 0, 0), font = font)
            x_start += 120
    elif x_start > 900:
        x_start = 30
        y_start += 50
        if hgt == 'C1-->D5':
            add_text.text((x_start, y_start), hgt, (225, 0, 0), font = font)
            x_start += 120
        else:
            add_text.text((x_start, y_start), hgt, (0, 0, 0), font = font)
            x_start += 120

background.save("/Users/weizhisong/Desktop/a_test.png")



from bs4 import BeautifulSoup


# html_doc = """
# <html><head><title>The Dormouse's story</title></head>
# <body>
# <p class="title"><b>The Dormouse's story</b></p>
#
# <p class="story">Once upon a time there were three little sisters; and their names were
# <a href="http://example.com/elsie" class="sister" id="link1">Elsie</a>,
# <a href="http://example.com/lacie" class="sister" id="link2">Lacie</a> and
# <a href="http://example.com/tillie" class="sister" id="link3">Tillie</a>;
# and they lived at the bottom of a well.</p>
#
# <p class="story">...</p>
# """
#
# soup = BeautifulSoup(html_doc, 'html.parser')
#
# print(soup.title)
# print(soup.title.string)
# print(soup.title.text)
# print(soup.p)
# print(soup.p['class'])
# print(soup.a)
# print(soup.find_all('a'))
# print(soup.find(id="link1"))
# print(soup.find(id="link2"))
# print(soup.find(id="link3"))
# print(soup.get_text())
#
#
# soup = BeautifulSoup('<b class="boldest" id="DemoID" > Extremely bold </b>', features='lxml')
# print(soup.b)
# print(soup.b.name)
# print(soup.b['class'])
# print(soup.b.get('class'))
# print(soup.b['id'])
#
#
# html_doc = """
# <html><head><title>The Dormouse's story</title></head>
# <body>
# <p class="title"><b>The Dormouse's story</b></p>
#
# <p class="story">Once upon a time there were three little sisters; and their names were
# <a href="http://example.com/elsie" class="sister" id="link1">Elsie</a>,
# <a href="http://example.com/lacie" class="sister" id="link2">Lacie</a> and
# <a href="http://example.com/tillie" class="sister" id="link3">Tillie</a>;
# and they lived at the bottom of a well.</p>
#
# <p class="story">...</p>
# """
#
# from bs4 import BeautifulSoup
# soup = BeautifulSoup(html_doc, 'html.parser')
#
# print(soup.head)
# print(soup.title)
# print(soup.title.text)
# print(soup.body)
# print(soup.body.b)
# print(soup.a)
# print()




test_html = '''
<tr><th class="th40" align="left" valign="top" style="border-color:#000; border-width: 1px 0px 0px 1px; border-style: solid"><nobr>Other DBs</nobr></th>
<td class="td40" style="border-color:#000; border-width: 1px 1px 0px 1px; border-style: solid"><table style="border:0;border-collapse:collapse;"><tr><td valign="top"><nobr>COG:&nbsp;</nobr></td><td><a href="https://www.ncbi.nlm.nih.gov/Structure/cdd/cddsrv.cgi?uid=COG5499">COG5499</a></td></tr></table></td></tr>
'''

soup = BeautifulSoup(test_html, features="lxml")


print()
print(soup.th.text)
print(soup.td.text)

print(soup.find_all('th'))

print(soup.th)






from Bio.Blast import NCBIXML


def blast_xml_parser(blast_result_xml, blast_result_with_description):

    # columns in output file
    # query	subject	identity	length	evalue	bitscore	subject_description

    blast_result_with_description_handle = open(blast_result_with_description, 'w')
    blast_result_with_description_handle.write('query\tsubject\tidentity\tlength\tevalue\tbitscore\tsubject_description\n')
    for blast_record in NCBIXML.parse(open(blast_result_xml)):
        query_id = blast_record.query.split(' ')[0]
        for alignment in blast_record.alignments:
            subject_id = alignment.hit_id
            subject_def = ' '.join(alignment.hit_def.split(' ')[1:])
            for hsp in alignment.hsps:
                identity = float("{0:.3f}".format(hsp.identities*100/hsp.align_length))
                aln_len = hsp.align_length
                e_value = hsp.expect
                bit_score = hsp.bits
                blast_result_with_description_handle.write('%s\t%s\t%s\t%s\t%s\t%s\t%s\n' % (query_id, subject_id, identity, aln_len, e_value, bit_score, subject_def))
    blast_result_with_description_handle.close()


blast_result_xml         = '/Users/songweizhi/Desktop/query_blastp_xml.tab'
blast_result_xml_summary = '/Users/songweizhi/Desktop/query_blastp_xml_summary.tab'
blast_xml_parser(blast_result_xml, blast_result_xml_summary)





import os
from ete3 import Tree
from ete3 import TreeStyle
from ete3 import NodeStyle
from ete3 import TextFace


def plot_tree(tree, tree_title, tree_output):
    # set tree parameters
    ts = TreeStyle()
    ts.mode = "r"  # tree model: 'r' for rectangular, 'c' for circular
    ts.show_leaf_name = 0
    # set tree title text parameters
    ts.title.add_face(TextFace(tree_title,
                               fsize = 8,
                               fgcolor = 'black',
                               ftype = 'Arial',
                               tight_text = False),
                      column = 0)  # tree title text setting
    # set layout parameters
    ts.rotation = 0  # from 0 to 360
    ts.show_scale = False
    ts.margin_top = 10  # top tree image margin
    ts.margin_bottom = 10  # bottom tree image margin
    ts.margin_left = 10  # left tree image margin
    ts.margin_right = 10  # right tree image margin
    ts.show_border = False  # set tree image border
    ts.branch_vertical_margin = 3  # 3 pixels between adjancent branches

    # set tree node style
    for each_node in tree.traverse():
        # leaf node parameters
        if each_node.is_leaf():
            ns = NodeStyle()
            ns["shape"] = "circle"  # dot shape: circle, square or sphere
            ns["size"] = 0  # dot size
            ns['hz_line_width'] = 0.5  # branch line width
            ns['vt_line_width'] = 0.5  # branch line width
            ns['hz_line_type'] = 0  # branch line type: 0 for solid, 1 for dashed, 2 for dotted
            ns['vt_line_type'] = 0  # branch line type
            ns["fgcolor"] = "blue"  # the dot setting
            each_node.add_face(TextFace(each_node.name,
                                        fsize = 5,
                                        fgcolor = 'black',
                                        tight_text = False,
                                        bold = False),
                               column = 0,
                               position = 'branch-right')  # leaf node the node name text setting

            each_node.set_style(ns)

        # non-leaf node parameters
        else:
            nlns = NodeStyle()
            nlns["size"] = 0  # dot size
            #nlns["rotation"] = 45
            each_node.add_face(TextFace(each_node.name,

                                        fsize = 3,
                                        fgcolor = 'black',
                                        tight_text = False,
                                        bold = False),
                               column = 5,
                               position = 'branch-top')  # non-leaf node name text setting)

            each_node.set_style(nlns)

    tree.render(tree_output, w=900, units="px", tree_style=ts)  # set figures size


#os.chdir('/Users/songweizhi/Desktop')

tree_2 = '(CF_Refined_71:0.21847,CF_Refined_170:0.41504,(((CF_Refined_7:0.63495,CF_Refined_96:0.68718)0.984:0.33915,CF_Refined_82:0.16074)0.980:0.12012,((CF_Refined_25:0.20437,CF_Refined_95:1.40476)0.367:0.60450,(((CF_Refined_86:0.37933,(CF_Refined_74:0.61406,CF_Refined_43:0.10850)1.000:0.34468)0.999:0.19175,((CF_Refined_57:0.19003,(CF_Refined_99:0.18534,CF_Refined_160:0.33153)0.861:0.04660)1.000:0.18553,(CF_Refined_78:0.64747,(CF_Refined_129:0.26317,(CF_Refined_64:0.25293,CF_Refined_100:0.10449)0.993:0.14949)0.577:0.13006)1.000:0.19231)0.998:0.16057)0.961:0.08413,((CF_Refined_155:0.18262,CF_Refined_180:0.02711)1.000:0.42318,(CF_Refined_162:0.64092,CF_Refined_131:0.36385)1.000:0.48656)0.992:0.23471)0.853:0.05581)0.955:0.08666)1.000:0.14706);'

tree = Tree(tree_2, format=1)


plot_tree(tree, 'Species_Tree', '/Users/songweizhi/Desktop/Species_Tree.png')




from requests import get
from bs4 import BeautifulSoup


# https://www.dataquest.io/blog/web-scraping-beautifulsoup/

url = 'http://www.imdb.com/search/title?release_date=2017&sort=num_votes,desc&page=1'
html_soup = BeautifulSoup(get(url).text, 'html.parser')
print(html_soup)

movie_containers = html_soup.find_all('div', class_='lister-item mode-advanced')


# Extract data
print('Year\tIMDB\tM_Score\tVote\tName')
for movie_container in movie_containers:

    # If the movie has Metascore, then extract:
    if movie_container.find('div', class_='ratings-metascore') is not None:
        name =    movie_container.h3.a.text
        year =    movie_container.h3.find('span', class_='lister-item-year').text
        imdb =    movie_container.strong.text
        m_score = movie_container.find('span', class_='metascore').text
        vote =    movie_container.find('span', attrs={'name': 'nv'})['data-value']

        print('%s\t%s\t%s\t%s\t%s' % (year, imdb, m_score, vote, name))




import os
import glob
import argparse
from Bio import SeqIO
import multiprocessing as mp
from datetime import datetime
from Bio.SeqRecord import SeqRecord
from BioSAK.global_functions import time_format
from BioSAK.global_functions import force_create_folder
from BioSAK.global_functions import sep_path_basename_ext
from BioSAK.global_functions import AnnotateNorm
from BioSAK.global_functions import get_gene_list_TotalDepth


COG2014_parser_usage = '''
===================================== COG2014 example commands =====================================

# module needed
module load python/3.7.3
module load blast+/2.6.0
module load diamond/0.9.24

# annotate protein sequences
BioSAK COG2014 -m P -t 6 -db_dir /srv/scratch/z5039045/DB/COG2014 -i recipient.faa 
BioSAK COG2014 -m P -t 6 -db_dir /srv/scratch/z5039045/DB/COG2014 -i faa_files -x faa -depth depth_files

# annotate DNA sequences (ORFs)
BioSAK COG2014 -m N -t 6 -db_dir /srv/scratch/z5039045/DB/COG2014 -i recipient.ffn -depth recipient.depth
BioSAK COG2014 -m N -t 6 -db_dir /srv/scratch/z5039045/DB/COG2014 -i ffn_files -x ffn

# Depth file format (one gene per line, tab separated)
gene_1	30
gene_2	10.58

# Prepare DB files:
cd $db_dir
wget ftp://ftp.ncbi.nlm.nih.gov//pub/COG/COG2014/data/prot2003-2014.fa.gz
wget ftp://ftp.ncbi.nlm.nih.gov//pub/COG/COG2014/data/prot2003-2014.tab
wget ftp://ftp.ncbi.nlm.nih.gov//pub/COG/COG2014/data/cog2003-2014.csv
wget ftp://ftp.ncbi.nlm.nih.gov//pub/COG/COG2014/data/cognames2003-2014.tab
wget ftp://ftp.ncbi.nlm.nih.gov//pub/COG/COG2014/data/fun2003-2014.tab        
gunzip prot2003-2014.fa.gz

# How it works:
1. COG2014 module uses Blast+/Diamond to get the best hits of query genes in the database 
   with users defined e-value cutoff (default 0.001).
2. The TotalDepth of a COG is calculated by summing up the depth of all genes assigned to it.
3. The percentage of GeneNumber/TotalDepth of genes assigned to a COG is calculated by dividing them 
   by the total number/depth of genes with COG assignment (default) or all query genes in a file (if "-pct_by_all" specified). 

# Note!!!
If you run COG2014 for multiple files in a batch manner and want to have their depth info incorporated into the results, 
you need to provide a folder containing individual depth files for each of your input sequence file.
Name of the depth file needs to be exactly the same as its corresponding sequence file, except the extension is ".depth".

====================================================================================================
'''


def dna2aa(dna_file, aa_file):
    query_aa_handle = open(aa_file, 'w')
    for each in SeqIO.parse(dna_file, 'fasta'):
        each_aa = each.seq.translate()
        each_aa_record = SeqRecord(each_aa)
        each_aa_record.id = each.id
        each_aa_record.description = each.description
        SeqIO.write(each_aa_record, query_aa_handle, 'fasta')
    query_aa_handle.close()


def best_hit(args):

    file_in = args['i']
    file_out = args['o']

    file_out_handle = open(file_out, 'w')
    best_hit_line = ''
    best_hit_query_id = ''
    best_hit_score = 0
    for blast_hit in open(file_in):
        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]
        bit_score = float(blast_hit_split[11])

        if best_hit_query_id == '':
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

        elif (query_id == best_hit_query_id) and (bit_score > best_hit_score):
            best_hit_score = bit_score
            best_hit_line = blast_hit

        elif query_id != best_hit_query_id:
            file_out_handle.write(best_hit_line)
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

    file_out_handle.write(best_hit_line)
    file_out_handle.close()


def COG2014_worker(argument_list):

    pwd_input_file =                    argument_list[0]
    pwd_prot2003_2014 =                 argument_list[1]
    refseq_acc_to_protein_id_dict =     argument_list[2]
    protein_id_to_cog_id_dict =         argument_list[3]
    cog_id_to_category_dict =           argument_list[4]
    cog_id_to_description_dict =        argument_list[5]
    cog_category_list =                 argument_list[6]
    cog_category_to_description_dict =  argument_list[7]
    sequence_type =                     argument_list[8]
    output_folder =                     argument_list[9]
    thread_num =                        argument_list[10]
    run_diamond =                       argument_list[11]
    evalue_cutoff =                     argument_list[12]
    depth_file =                        argument_list[13]
    pct_by_all =                        argument_list[14]

    input_seq_no_path, input_seq_no_ext, input_seq_ext = sep_path_basename_ext(pwd_input_file)
    current_output_folder = '%s/%s_COG2014_wd' % (output_folder, input_seq_no_ext)

    pwd_blastp_output =                        '%s/%s_blastp.tab'                             % (current_output_folder, input_seq_no_ext)
    pwd_blastp_output_besthits =               '%s/%s_blastp_besthits.tab'                    % (current_output_folder, input_seq_no_ext)
    pwd_blastp_output_besthits_no_gi =         '%s/%s_blastp_besthits_no_gi.tab'              % (current_output_folder, input_seq_no_ext)
    pwd_query_to_cog_txt =                     '%s/%s_query_to_cog.txt'                       % (current_output_folder, input_seq_no_ext)

    pwd_cog_stats_GeneNumber =                 '%s/%s_cog_stats_GeneNumber.txt'               % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_TotalDepth =                 '%s/%s_cog_stats_TotalDepth.txt'               % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_GeneNumber =                '%s/%s_func_stats_GeneNumber.txt'              % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_TotalDepth =                '%s/%s_func_stats_TotalDepth.txt'              % (current_output_folder, input_seq_no_ext)

    pwd_cog_stats_GeneNumber_pct =             '%s/%s_cog_stats_GeneNumber_pct.txt'           % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_TotalDepth_pct =             '%s/%s_cog_stats_TotalDepth_pct.txt'           % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_GeneNumber_pct =            '%s/%s_func_stats_GeneNumber_pct.txt'          % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_TotalDepth_pct =            '%s/%s_func_stats_TotalDepth_pct.txt'          % (current_output_folder, input_seq_no_ext)

    pwd_cog_stats_GeneNumber_pct_by_all =      '%s/%s_cog_stats_GeneNumber_pct_by_all.txt'    % (current_output_folder, input_seq_no_ext)
    pwd_cog_stats_TotalDepth_pct_by_all =      '%s/%s_cog_stats_TotalDepth_pct_by_all.txt'    % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_GeneNumber_pct_by_all =     '%s/%s_func_stats_GeneNumber_pct_by_all.txt'   % (current_output_folder, input_seq_no_ext)
    pwd_func_stats_TotalDepth_pct_by_all =     '%s/%s_func_stats_TotalDepth_pct_by_all.txt'   % (current_output_folder, input_seq_no_ext)

    force_create_folder(current_output_folder)

    input_seq_aa = ''
    if sequence_type in ['N', 'n']:
        input_seq_aa = '%s_aa.fasta' % input_seq_no_ext
        dna2aa(pwd_input_file, input_seq_aa)
    elif sequence_type in ['P', 'p']:
        input_seq_aa = pwd_input_file
    else:
        print('Specified input sequence type unrecognizable, program exited!')
        exit()

    # run blastp
    if run_diamond is False:
        os.system('blastp -query %s -db %s -out %s -evalue %s -outfmt 6 -show_gis -num_threads %s' % (input_seq_aa, pwd_prot2003_2014, pwd_blastp_output, evalue_cutoff, thread_num))
    else:
        os.system('diamond blastp -q %s --db %s --out %s --evalue %s --outfmt 6 --threads %s --quiet' % (input_seq_aa, pwd_prot2003_2014, pwd_blastp_output, evalue_cutoff, thread_num))


    # keep only best hits
    best_hit({'i': pwd_blastp_output, 'o': pwd_blastp_output_besthits})

    # remove gi from best hits
    if run_diamond is True:
        pwd_blastp_output_besthits_no_gi_handle = open(pwd_blastp_output_besthits_no_gi, 'w')
        for each_hit in open(pwd_blastp_output_besthits):
            each_hit_split = each_hit.strip().split('\t')
            each_hit_subject = each_hit_split[1]
            each_hit_subject_no_gi = each_hit_subject.split('|')[-2]
            each_hit_split[1] = each_hit_subject_no_gi
            pwd_blastp_output_besthits_no_gi_handle.write('%s\n' % '\t'.join(each_hit_split))
        pwd_blastp_output_besthits_no_gi_handle.close()

        pwd_blastp_output_besthits = pwd_blastp_output_besthits_no_gi

    # get query_seq_to_refseq_acc_dict
    query_seq_to_refseq_acc_dict = {}
    for blastp_hit in open(pwd_blastp_output_besthits):
        blastp_hit_split = blastp_hit.strip().split('\t')
        query_seq_to_refseq_acc_dict[blastp_hit_split[0]] = blastp_hit_split[1].split('.')[0]

    # get query sequences list
    query_seq_list = []
    for query_seq in SeqIO.parse(pwd_input_file, 'fasta'):
        query_seq_list.append(query_seq.id)

    # export annotation
    cog_id_num_dict = {}
    cog_id_to_gene_member_dict = {}
    cog_cate_num_dict = {}
    cog_cate_to_gene_member_dict = {}
    genes_with_cog = set()
    pwd_query_to_cog_txt_handle = open(pwd_query_to_cog_txt, 'w')
    pwd_query_to_cog_txt_handle.write('Query\tCOG\tCategory\tDescription\n')
    for query_gene in sorted(query_seq_list):
        if query_gene in query_seq_to_refseq_acc_dict:
            refseq_acc = query_seq_to_refseq_acc_dict[query_gene]
            db_protein_id = refseq_acc_to_protein_id_dict[refseq_acc]

            if db_protein_id not in protein_id_to_cog_id_dict:
                pwd_query_to_cog_txt_handle.write('%s\n' % (query_gene))
            else:
                cog_id = protein_id_to_cog_id_dict[db_protein_id]
                cog_cate = cog_id_to_category_dict[cog_id]
                cog_des = cog_id_to_description_dict[cog_id]
                pwd_query_to_cog_txt_handle.write('%s\t%s\t%s\t%s\n' % (query_gene, cog_id, cog_cate, cog_des))
                genes_with_cog.add(query_gene)

                # update cog_id_num_dict
                if cog_id not in cog_id_num_dict:
                    cog_id_num_dict[cog_id] = 1
                    cog_id_to_gene_member_dict[cog_id] = [query_gene]
                else:
                    cog_id_num_dict[cog_id] += 1
                    cog_id_to_gene_member_dict[cog_id].append(query_gene)

                # update cog_cate_num_dict
                for each_cog_cate in cog_cate:
                    if each_cog_cate not in cog_cate_num_dict:
                        cog_cate_num_dict[each_cog_cate] = 1
                        cog_cate_to_gene_member_dict[each_cog_cate] = [query_gene]
                    else:
                        cog_cate_num_dict[each_cog_cate] += 1
                        cog_cate_to_gene_member_dict[each_cog_cate].append(query_gene)

        else:
            pwd_query_to_cog_txt_handle.write('%s\n' % (query_gene))

    pwd_query_to_cog_txt_handle.close()


    # read in depth info
    gene_depth_dict = {}
    if depth_file is not None:
        for each_depth in open(depth_file):
            each_depth_split = each_depth.strip().split('\t')
            gene_depth_dict[each_depth_split[0]] = float(each_depth_split[1])

    # get TotalDepth of all query genes or genes with cog assignment
    if depth_file is not None:
        genes_with_cog_TotalDepth = get_gene_list_TotalDepth(genes_with_cog, gene_depth_dict)
        total_depth_for_all_query_genes = get_gene_list_TotalDepth(query_seq_list, gene_depth_dict)

    #################### export cog_stats_GeneNumber ####################

    pwd_cog_stats_GeneNumber_handle = open(pwd_cog_stats_GeneNumber, 'w')
    pwd_cog_stats_GeneNumber_handle.write('COG\tGeneNumber\tDescription\n')
    for each_cog_id in cog_id_num_dict:
        each_cog_id_GeneNumber = cog_id_num_dict[each_cog_id]
        pwd_cog_stats_GeneNumber_handle.write('%s\t%s\t%s\n' % (each_cog_id, each_cog_id_GeneNumber, cog_id_to_description_dict[each_cog_id]))
    pwd_cog_stats_GeneNumber_handle.close()

    #################### export cog_stats_TotalDepth ####################

    if depth_file is not None:
        pwd_cog_stats_TotalDepth_handle = open(pwd_cog_stats_TotalDepth, 'w')
        pwd_cog_stats_TotalDepth_handle.write('COG\tTotalDepth\tDescription\n')
        for each_cog_id in cog_id_to_gene_member_dict:
            each_cog_id_gene_member = cog_id_to_gene_member_dict[each_cog_id]
            each_cog_id_TotalDepth = 0
            for each_gene in each_cog_id_gene_member:
                each_gene_depth = gene_depth_dict[each_gene]
                each_cog_id_TotalDepth += each_gene_depth
            each_cog_id_TotalDepth = float("{0:.2f}".format(each_cog_id_TotalDepth))
            pwd_cog_stats_TotalDepth_handle.write('%s\t%s\t%s\n' % (each_cog_id, each_cog_id_TotalDepth, cog_id_to_description_dict[each_cog_id]))
        pwd_cog_stats_TotalDepth_handle.close()

    #################### export func_stats_GeneNumber ####################

    pwd_func_stats_GeneNumber_handle = open(pwd_func_stats_GeneNumber, 'w')
    pwd_func_stats_GeneNumber_handle.write('Category\tGeneNumber\tDescription\n')
    for each_cog_cate in cog_category_list:
        each_cog_cate_GeneNumber = 0
        if each_cog_cate in cog_cate_num_dict:
            each_cog_cate_GeneNumber = cog_cate_num_dict[each_cog_cate]
        pwd_func_stats_GeneNumber_handle.write('%s\t%s\t%s\n' % (each_cog_cate, each_cog_cate_GeneNumber, cog_category_to_description_dict[each_cog_cate]))
    pwd_func_stats_GeneNumber_handle.close()

    #################### export func_stats_TotalDepth ####################

    if depth_file is not None:
        pwd_func_stats_TotalDepth_handle = open(pwd_func_stats_TotalDepth, 'w')
        pwd_func_stats_TotalDepth_handle.write('Category\tTotalDepth\tDescription\n')
        for each_cog_cate in cog_category_list:
            each_cog_cate_TotalDepth = 0
            if each_cog_cate in cog_cate_to_gene_member_dict:
                each_cog_cate_gene_member = cog_cate_to_gene_member_dict[each_cog_cate]
                for each_gene in each_cog_cate_gene_member:
                    each_gene_depth = gene_depth_dict[each_gene]
                    each_cog_cate_TotalDepth += each_gene_depth
            each_cog_cate_TotalDepth = float("{0:.2f}".format(each_cog_cate_TotalDepth))
            pwd_func_stats_TotalDepth_handle.write('%s\t%s\t%s\n' % (each_cog_cate, each_cog_cate_TotalDepth, cog_category_to_description_dict[each_cog_cate]))
        pwd_func_stats_TotalDepth_handle.close()

    #################### get pct files ####################

    AnnotateNorm(file_in=pwd_cog_stats_GeneNumber,  skip_header=True, value_column=2, Divisor_value=len(genes_with_cog), file_out=pwd_cog_stats_GeneNumber_pct,  file_out_header='Category\tGeneNumber_pct\tDescription\n')
    AnnotateNorm(file_in=pwd_func_stats_GeneNumber, skip_header=True, value_column=2, Divisor_value=len(genes_with_cog), file_out=pwd_func_stats_GeneNumber_pct, file_out_header='Category\tGeneNumber_pct\tDescription\n')
    if depth_file is not None:
        AnnotateNorm(file_in=pwd_cog_stats_TotalDepth,  skip_header=True, value_column=2, Divisor_value=genes_with_cog_TotalDepth, file_out=pwd_cog_stats_TotalDepth_pct,  file_out_header='Category\tTotalDepth_pct\tDescription\n')
        AnnotateNorm(file_in=pwd_func_stats_TotalDepth, skip_header=True, value_column=2, Divisor_value=genes_with_cog_TotalDepth, file_out=pwd_func_stats_TotalDepth_pct, file_out_header='Category\tTotalDepth_pct\tDescription\n')
    if pct_by_all is True:
        AnnotateNorm(file_in=pwd_cog_stats_GeneNumber,  skip_header=True, value_column=2, Divisor_value=len(query_seq_list), file_out=pwd_cog_stats_GeneNumber_pct_by_all,  file_out_header='Category\tGeneNumber_pct_by_all\tDescription\n')
        AnnotateNorm(file_in=pwd_func_stats_GeneNumber, skip_header=True, value_column=2, Divisor_value=len(query_seq_list), file_out=pwd_func_stats_GeneNumber_pct_by_all, file_out_header='Category\tGeneNumber_pct_by_all\tDescription\n')
        if depth_file is not None:
            AnnotateNorm(file_in=pwd_cog_stats_TotalDepth,  skip_header=True, value_column=2, Divisor_value=total_depth_for_all_query_genes, file_out=pwd_cog_stats_TotalDepth_pct_by_all,  file_out_header='Category\tTotalDepth_pct_by_all\tDescription\n')
            AnnotateNorm(file_in=pwd_func_stats_TotalDepth, skip_header=True, value_column=2, Divisor_value=total_depth_for_all_query_genes, file_out=pwd_func_stats_TotalDepth_pct_by_all, file_out_header='Category\tTotalDepth_pct_by_all\tDescription\n')


def get_COG_annot_df(annotation_dir, stats_level, annotation_df_absolute_num, annotation_df_percentage, annotation_df_percentage_by_all, with_depth, pct_by_all):

    annotation_dir_re = '%s/*_COG2014_wd' % annotation_dir
    annotation_folder_list = [os.path.basename(file_name) for file_name in glob.glob(annotation_dir_re)]

    cog_num_dict = {}
    cog_num_pct_dict = {}
    cog_num_pct_by_all_dict = {}
    all_identified_cog = set()
    for annotation_folder in annotation_folder_list:

        annotation_folder_basename = annotation_folder.split('_COG2014_wd')[0]

        pwd_annotation_stats_file = ''
        pwd_annotation_stats_file_pct = ''
        if stats_level == 'cog_id':
            if with_depth is False:
                pwd_annotation_stats_file =             '%s/%s/%s_cog_stats_GeneNumber.txt'                 % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct =         '%s/%s/%s_cog_stats_GeneNumber_pct.txt'             % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct_by_all =  '%s/%s/%s_cog_stats_GeneNumber_pct_by_all.txt'      % (annotation_dir, annotation_folder, annotation_folder_basename)
            else:
                pwd_annotation_stats_file =             '%s/%s/%s_cog_stats_TotalDepth.txt'                 % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct =         '%s/%s/%s_cog_stats_TotalDepth_pct.txt'             % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct_by_all =  '%s/%s/%s_cog_stats_TotalDepth_pct_by_all.txt'      % (annotation_dir, annotation_folder, annotation_folder_basename)

        if stats_level == 'cog_cate':
            if with_depth is False:
                pwd_annotation_stats_file =             '%s/%s/%s_func_stats_GeneNumber.txt'                % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct =         '%s/%s/%s_func_stats_GeneNumber_pct.txt'            % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct_by_all =  '%s/%s/%s_func_stats_GeneNumber_pct_by_all.txt'     % (annotation_dir, annotation_folder, annotation_folder_basename)
            else:
                pwd_annotation_stats_file =             '%s/%s/%s_func_stats_TotalDepth.txt'                % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct =         '%s/%s/%s_func_stats_TotalDepth_pct.txt'            % (annotation_dir, annotation_folder, annotation_folder_basename)
                pwd_annotation_stats_file_pct_by_all =  '%s/%s/%s_func_stats_TotalDepth_pct_by_all.txt'     % (annotation_dir, annotation_folder, annotation_folder_basename)

        current_cog_to_num_dict = {}
        for cog in open(pwd_annotation_stats_file):
            if (not cog.startswith('Category	')) and (not cog.startswith('COG	')):
                cog_split = cog.strip().split('\t')
                if with_depth is False:
                    current_cog_to_num_dict[cog_split[0]] = int(cog_split[1])
                else:
                    current_cog_to_num_dict[cog_split[0]] = float(cog_split[1])
                all_identified_cog.add(cog_split[0])
        cog_num_dict[annotation_folder_basename] = current_cog_to_num_dict


        current_cog_to_num_pct_dict = {}
        for cog in open(pwd_annotation_stats_file_pct):
            if (not cog.startswith('Category	')) and (not cog.startswith('COG	')):
                cog_split = cog.strip().split('\t')
                current_cog_to_num_pct_dict[cog_split[0]] = float(cog_split[1])
                all_identified_cog.add(cog_split[0])
        cog_num_pct_dict[annotation_folder_basename] = current_cog_to_num_pct_dict


        if pct_by_all is True:
            current_cog_to_num_pct_by_all_dict = {}
            for cog in open(pwd_annotation_stats_file_pct_by_all):
                if (not cog.startswith('Category	')) and (not cog.startswith('COG	')):
                    cog_split = cog.strip().split('\t')
                    current_cog_to_num_pct_by_all_dict[cog_split[0]] = float(cog_split[1])
                    all_identified_cog.add(cog_split[0])
            cog_num_pct_by_all_dict[annotation_folder_basename] = current_cog_to_num_pct_by_all_dict


    all_identified_cog_list = sorted([i for i in all_identified_cog])


    annotation_df_absolute_num_handle = open(annotation_df_absolute_num, 'w')
    annotation_df_percentage_handle = open(annotation_df_percentage, 'w')
    annotation_df_absolute_num_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list))
    annotation_df_percentage_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list))
    for annotation_folder in sorted(annotation_folder_list):

        annotation_folder_basename = annotation_folder.split('_COG2014_wd')[0]
        current_cog_num_dict = cog_num_dict[annotation_folder_basename]
        current_cog_num_pct_dict = cog_num_pct_dict[annotation_folder_basename]

        current_cog_num_list = []
        current_cog_num_list_pct = []
        for identified_cog in all_identified_cog_list:

            # get num list
            identified_cog_num = 0
            identified_cog_num_pct = 0
            if identified_cog in current_cog_num_dict:
                identified_cog_num = current_cog_num_dict[identified_cog]
                identified_cog_num_pct = current_cog_num_pct_dict[identified_cog]

            current_cog_num_list.append(identified_cog_num)
            current_cog_num_list_pct.append(identified_cog_num_pct)

        # write out
        annotation_df_absolute_num_handle.write('%s\t%s\n' % (annotation_folder_basename, '\t'.join([str(i) for i in current_cog_num_list])))
        annotation_df_percentage_handle.write('%s\t%s\n' % (annotation_folder_basename, '\t'.join([str(i) for i in current_cog_num_list_pct])))

    annotation_df_absolute_num_handle.close()
    annotation_df_percentage_handle.close()

    ########## write out pct_by_all_file ##########

    if pct_by_all is True:
        annotation_df_percentage_by_all_handle = open(annotation_df_percentage_by_all, 'w')
        annotation_df_percentage_by_all_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list))
        for annotation_folder in sorted(annotation_folder_list):

            annotation_folder_basename = annotation_folder.split('_COG2014_wd')[0]
            current_cog_num_pct_by_all_dict = cog_num_pct_by_all_dict[annotation_folder_basename]

            current_cog_num_list_pct_by_all = []
            for identified_cog in all_identified_cog_list:

                # get num list
                identified_cog_num_pct_by_all = 0
                if identified_cog in current_cog_num_pct_by_all_dict:
                    identified_cog_num_pct_by_all = current_cog_num_pct_by_all_dict[identified_cog]

                current_cog_num_list_pct_by_all.append(identified_cog_num_pct_by_all)

            # write out
            annotation_df_percentage_by_all_handle.write('%s\t%s\n' % (annotation_folder_basename, '\t'.join([str(i) for i in current_cog_num_list_pct_by_all])))

        annotation_df_percentage_by_all_handle.close()


def COG2014(args):

    file_in =           args['i']
    file_extension =    args['x']
    sequence_type =     args['m']
    depth_file =        args['depth']
    pct_by_all =        args['pct_by_all']
    DB_dir =            args['db_dir']
    num_threads =       args['t']
    run_diamond =       args['diamond']
    evalue_cutoff =     args['evalue']

    pwd_prot2003_2014 =         '%s/prot2003-2014.fa'         % DB_dir
    pwd_prot2003_2014_diamond = '%s/prot2003-2014.fa.dmnd'    % DB_dir
    pwd_prot2003_2014_tab =     '%s/prot2003-2014.tab'        % DB_dir
    pwd_cog2003_2014 =          '%s/cog2003-2014.csv'         % DB_dir
    pwd_cognames2003_2014 =     '%s/cognames2003-2014.tab'    % DB_dir
    pwd_fun2003_2014 =          '%s/fun2003-2014.tab'         % DB_dir


    ############################################ check whether db file exist ###########################################

    # check whether db file exist
    unfound_inputs = []
    for each_input in [pwd_prot2003_2014, pwd_cognames2003_2014, pwd_fun2003_2014]:
        if (not os.path.isfile(each_input)) and (not os.path.isdir(each_input)):
            unfound_inputs.append(each_input)
    if len(unfound_inputs) > 0:
        for each_unfound in unfound_inputs:
            print('%s not found' % each_unfound)
        exit()

    # check whether db index esixt
    if run_diamond is False:
        unfound_db_index_file = []
        for db_index in ['phr', 'pin', 'pnd', 'pni', 'pog', 'psd', 'psi', 'psq']:
            pwd_db_index = '%s/prot2003-2014.fa.%s' % (DB_dir, db_index)
            if not os.path.isfile(pwd_db_index):
                unfound_db_index_file.append(db_index)
        if len(unfound_db_index_file) > 0:
            print(datetime.now().strftime(time_format) + 'DB index files not found, runing makeblastdb first')
            makeblastdb_cmd = 'makeblastdb -in %s -dbtype prot -parse_seqids -logfile %s.log' % (pwd_prot2003_2014, pwd_prot2003_2014)
            os.system(makeblastdb_cmd)
            print(datetime.now().strftime(time_format) + 'makeblastdb finished')

    if run_diamond is True:
        if os.path.isfile(pwd_prot2003_2014_diamond) is False:
            print(datetime.now().strftime(time_format) + 'DB file not found, making diamond db')
            diamond_makedb_cmd = 'diamond makedb --in %s --db %s --quiet' % (pwd_prot2003_2014, pwd_prot2003_2014_diamond)
            os.system(diamond_makedb_cmd)


    ################################################# read db into dict ################################################

    # get refseq_acc_to_protein_id_dict (prot2003-2014.tab)
    refseq_acc_to_protein_id_dict = {}
    for refseq_acc in open(pwd_prot2003_2014_tab):
        refseq_acc_split = refseq_acc.strip().split('\t')
        refseq_acc_to_protein_id_dict[refseq_acc_split[1]] = refseq_acc_split[0]

    # get protein_id_to_cog_id_dict (cog2003-2014.csv)
    protein_id_to_cog_id_dict = {}
    for protein_to_cog in open(pwd_cog2003_2014):
        protein_to_cog_split = protein_to_cog.strip().split(',')
        protein_id = protein_to_cog_split[2]
        cog_id = protein_to_cog_split[6]
        protein_id_to_cog_id_dict[protein_id] = cog_id

    # get cog_id_to_category_dict and cog_id_to_description_dict (cognames2003-2014.tab)
    cog_id_to_category_dict = {}
    cog_id_to_description_dict = {}
    for cog_id_to_cate_des in open(pwd_cognames2003_2014, encoding='windows-1252'):
        if not cog_id_to_cate_des.startswith('#'):
            cog_id_to_cate_des_split = cog_id_to_cate_des.strip().split('\t')
            cog_id = cog_id_to_cate_des_split[0]
            cog_cate = cog_id_to_cate_des_split[1]
            cog_des = cog_id_to_cate_des_split[2]
            cog_id_to_category_dict[cog_id] = cog_cate
            cog_id_to_description_dict[cog_id] = cog_des

    # get cog_category_to_description_dict (fun2003-2014.tab)
    cog_category_list = []
    cog_category_to_description_dict = {}
    for cog_category in open(pwd_fun2003_2014):
        if not cog_category.startswith('#'):
            cog_category_split = cog_category.strip().split('\t')
            cog_category_list.append(cog_category_split[0])
            cog_category_to_description_dict[cog_category_split[0]] = cog_category_split[1]


    ################################################## if input is file ################################################

    # if input is file
    if os.path.isfile(file_in) is True:

        # check whether depth file exist
        if depth_file is not None:
            if os.path.isfile(depth_file) is False:
                print(datetime.now().strftime(time_format) + 'specified depth file not found, program exited!')
                exit()

        print(datetime.now().strftime(time_format) + 'Running COG annotation for 1 file with %s cores' % (num_threads))

        file_in_path, file_in_basename, file_in_ext = sep_path_basename_ext(file_in)

        COG2014_worker([file_in,
                        pwd_prot2003_2014,
                        refseq_acc_to_protein_id_dict,
                        protein_id_to_cog_id_dict,
                        cog_id_to_category_dict,
                        cog_id_to_description_dict,
                        cog_category_list,
                        cog_category_to_description_dict,
                        sequence_type,
                        file_in_path,
                        num_threads,
                        run_diamond,
                        evalue_cutoff,
                        depth_file,
                        pct_by_all])


    ################################################ if input is folder ################################################

    # if input is folder
    else:

        # check whether input folder exist
        if os.path.isdir(file_in) is False:
            print(datetime.now().strftime(time_format) + 'input folder not found, program exited')
            exit()

        else:
            # check whether input genome exist
            input_file_re = '%s/*.%s' % (file_in, file_extension)
            input_file_name_list = [os.path.basename(file_name) for file_name in glob.glob(input_file_re)]

            if len(input_file_name_list) == 0:
                print(datetime.now().strftime(time_format) + 'input file not found, program exited')
                exit()

            # check whether depth file exist
            if depth_file is not None:

                if os.path.isfile(depth_file) is True:
                    print(datetime.now().strftime(time_format) + 'please provide the folder containing individual depth files (with extension .depth) for each of your input sequence file.')
                    print(datetime.now().strftime(time_format) + 'a single file (not folder) detected, program exited!')
                    exit()

                if os.path.isdir(depth_file) is False:
                    print(datetime.now().strftime(time_format) + 'provided depth folder not found, program exited!')
                    exit()

                if os.path.isdir(depth_file) is True:

                    undetected_depth_file = []
                    for input_seq_file in input_file_name_list:
                        input_seq_file_basename = '.'.join(input_seq_file.split('.')[:-1])
                        input_seq_file_depth = '%s/%s.depth' % (depth_file, input_seq_file_basename)
                        if os.path.isfile(input_seq_file_depth) is False:
                            undetected_depth_file.append(input_seq_file_depth)

                    if len(undetected_depth_file) > 0:
                        print(datetime.now().strftime(time_format) + 'the following depth files not found, program exited!')
                        print(','.join(undetected_depth_file))
                        exit()


            ################################################### define file name ###################################################

            if '/' in file_in:
                file_in_folder_name = file_in.split('/')[-1]
            else:
                file_in_folder_name = file_in

            output_folder =                             '%s_COG2014_wd'                             % file_in_folder_name

            # create output folder
            force_create_folder(output_folder)


            ######################################################### main #########################################################

            print(datetime.now().strftime(time_format) + 'Running COG annotation for %s files with %s cores' % (len(input_file_name_list), num_threads))

            list_for_multiple_arguments_COG = []
            for input_file in input_file_name_list:

                input_file_basename = '.'.join(input_file.split('.')[:-1])
                pwd_input_file = '%s/%s' % (file_in, input_file)


                # get path to current depth file
                if depth_file is None:
                    input_file_depth = None
                else:
                    input_file_depth = '%s/%s.depth' % (depth_file, input_file_basename)

                list_for_multiple_arguments_COG.append([pwd_input_file,
                                                        pwd_prot2003_2014,
                                                        refseq_acc_to_protein_id_dict,
                                                        protein_id_to_cog_id_dict,
                                                        cog_id_to_category_dict,
                                                        cog_id_to_description_dict,
                                                        cog_category_list,
                                                        cog_category_to_description_dict,
                                                        sequence_type,
                                                        output_folder,
                                                        1,
                                                        run_diamond,
                                                        evalue_cutoff,
                                                        input_file_depth,
                                                        pct_by_all])

            # run COG annotaion files with multiprocessing
            pool = mp.Pool(processes=num_threads)
            pool.map(COG2014_worker, list_for_multiple_arguments_COG)
            pool.close()
            pool.join()

            ######################################################### get dataframe #########################################################

            annotation_df_cog_cate_GeneNumber =             '%s/%s_COG2014_cate_GeneNumber.txt'             % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_GeneNumber_pct =         '%s/%s_COG2014_cate_GeneNumber_pct.txt'         % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_GeneNumber_pct_by_all =  '%s/%s_COG2014_cate_GeneNumber_pct_by_all.txt'  % (output_folder, file_in_folder_name)

            annotation_df_cog_cate_TotalDepth =             '%s/%s_COG2014_cate_TotalDepth.txt'             % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_TotalDepth_pct =         '%s/%s_COG2014_cate_TotalDepth_pct.txt'         % (output_folder, file_in_folder_name)
            annotation_df_cog_cate_TotalDepth_pct_by_all =  '%s/%s_COG2014_cate_TotalDepth_pct_by_all.txt'  % (output_folder, file_in_folder_name)

            annotation_df_cog_id_GeneNumber =               '%s/%s_COG2014_id_GeneNumber.txt'               % (output_folder, file_in_folder_name)
            annotation_df_cog_id_GeneNumber_pct =           '%s/%s_COG2014_id_GeneNumber_pct.txt'           % (output_folder, file_in_folder_name)
            annotation_df_cog_id_GeneNumber_pct_by_all =    '%s/%s_COG2014_id_GeneNumber_pct_by_all.txt'    % (output_folder, file_in_folder_name)

            annotation_df_cog_id_TotalDepth =               '%s/%s_COG2014_id_TotalDepth.txt'               % (output_folder, file_in_folder_name)
            annotation_df_cog_id_TotalDepth_pct =           '%s/%s_COG2014_id_TotalDepth_pct.txt'           % (output_folder, file_in_folder_name)
            annotation_df_cog_id_TotalDepth_pct_by_all =    '%s/%s_COG2014_id_TotalDepth_pct_by_all.txt'    % (output_folder, file_in_folder_name)

            print(datetime.now().strftime(time_format) + 'Data matrix exported to:')

            # get df
            get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_GeneNumber, annotation_df_cog_cate_GeneNumber_pct, annotation_df_cog_cate_GeneNumber_pct_by_all, with_depth=False, pct_by_all=False)
            get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_GeneNumber, annotation_df_cog_id_GeneNumber_pct, annotation_df_cog_id_GeneNumber_pct_by_all, with_depth=False, pct_by_all=False)
            if pct_by_all is True:
                get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_GeneNumber, annotation_df_cog_cate_GeneNumber_pct, annotation_df_cog_cate_GeneNumber_pct_by_all, with_depth=False, pct_by_all=True)
                get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_GeneNumber, annotation_df_cog_id_GeneNumber_pct, annotation_df_cog_id_GeneNumber_pct_by_all, with_depth=False, pct_by_all=True)

            # report
            if pct_by_all is False:
                print(datetime.now().strftime(time_format) + '%s and %s' % (annotation_df_cog_id_GeneNumber.split('/')[-1], annotation_df_cog_id_GeneNumber_pct.split('/')[-1]))
                print(datetime.now().strftime(time_format) + '%s and %s' % (annotation_df_cog_cate_GeneNumber.split('/')[-1], annotation_df_cog_cate_GeneNumber_pct.split('/')[-1]))
            else:
                print(datetime.now().strftime(time_format) + '%s, %s and %s' % (annotation_df_cog_id_GeneNumber.split('/')[-1], annotation_df_cog_id_GeneNumber_pct.split('/')[-1], annotation_df_cog_id_GeneNumber_pct_by_all.split('/')[-1]))
                print(datetime.now().strftime(time_format) + '%s, %s and %s' % (annotation_df_cog_cate_GeneNumber.split('/')[-1], annotation_df_cog_cate_GeneNumber_pct.split('/')[-1], annotation_df_cog_cate_GeneNumber_pct_by_all.split('/')[-1]))

            if depth_file is not None:
                get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_TotalDepth, annotation_df_cog_cate_TotalDepth_pct, annotation_df_cog_cate_TotalDepth_pct_by_all, with_depth=True, pct_by_all=False)
                get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_TotalDepth, annotation_df_cog_id_TotalDepth_pct, annotation_df_cog_id_TotalDepth_pct_by_all, with_depth=True, pct_by_all=False)
                if pct_by_all is True:
                    get_COG_annot_df(output_folder, 'cog_cate', annotation_df_cog_cate_TotalDepth, annotation_df_cog_cate_TotalDepth_pct, annotation_df_cog_cate_TotalDepth_pct_by_all, with_depth=True, pct_by_all=True)
                    get_COG_annot_df(output_folder, 'cog_id', annotation_df_cog_id_TotalDepth, annotation_df_cog_id_TotalDepth_pct, annotation_df_cog_id_TotalDepth_pct_by_all, with_depth=True, pct_by_all=True)

                # report
                if pct_by_all is False:
                    print(datetime.now().strftime(time_format) + '%s and %s' % (annotation_df_cog_id_TotalDepth.split('/')[-1], annotation_df_cog_id_TotalDepth_pct.split('/')[-1]))
                    print(datetime.now().strftime(time_format) + '%s and %s' % (annotation_df_cog_cate_TotalDepth.split('/')[-1], annotation_df_cog_cate_TotalDepth_pct.split('/')[-1]))
                else:
                    print(datetime.now().strftime(time_format) + '%s, %s and %s' % (annotation_df_cog_id_TotalDepth.split('/')[-1], annotation_df_cog_id_TotalDepth_pct.split('/')[-1], annotation_df_cog_id_TotalDepth_pct_by_all.split('/')[-1]))
                    print(datetime.now().strftime(time_format) + '%s, %s and %s' % (annotation_df_cog_cate_TotalDepth.split('/')[-1], annotation_df_cog_cate_TotalDepth_pct.split('/')[-1], annotation_df_cog_cate_TotalDepth_pct_by_all.split('/')[-1]))

    ################################################## Final report ####################################################

    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == '__main__':

    COG_parser = argparse.ArgumentParser()

    # arguments for COG_parser
    COG_parser.add_argument('-i',               required=True,                              help='path to input sequences (in multi-fasta format)')
    COG_parser.add_argument('-x',               required=False,                             help='file extension')
    COG_parser.add_argument('-m',               required=True,                              help='sequence type, "N/n" for "nucleotide", "P/p" for "protein"')
    COG_parser.add_argument('-depth',           required=False, default=None,               help='gene depth file/folder')
    COG_parser.add_argument('-pct_by_all',      required=False, action='store_true',        help='normalize by all query genes, rather than those with COG assignment')
    COG_parser.add_argument('-db_dir',          required=True,                              help='DB folder')
    COG_parser.add_argument('-diamond',         required=False, action='store_true',        help='run diamond (for big dataset), default is NCBI blastp')
    COG_parser.add_argument('-t',               required=False, type=int, default=1,        help='number of threads')
    COG_parser.add_argument('-evalue',          required=False, default=0.001, type=float,  help='evalue cutoff, default: 0.001')

    args = vars(COG_parser.parse_args())

    COG2014(args)






def get_pwy_ec_cutoff_by_mag_cpl(mag_cpl, mag_cpl_to_cutoff_dict, mag_cpl_threshold_list):

    mag_cpl_threshold_list_high_to_low = sorted(mag_cpl_threshold_list)[::-1]

    cutoffs_determined = False
    pathway_cutoff = 0
    key_enzyme_cutoff = 0
    for cpl_threshold in mag_cpl_threshold_list_high_to_low:

        if cutoffs_determined is False:
            if mag_cpl >= cpl_threshold:
                key_enzyme_cutoff = mag_cpl_to_cutoff_dict[cpl_threshold][0]
                pathway_cutoff = mag_cpl_to_cutoff_dict[cpl_threshold][1]
                cutoffs_determined = True

    return key_enzyme_cutoff, pathway_cutoff


# cut-offs in the last two columns will be used if MAG completeness no less than the value specified in the first column.
mag_completeness = '/Users/songweizhi/Desktop/mag_completeness.txt'
cutoff_table     = '/Users/songweizhi/Desktop/cutoff_table.txt'


# read in cutoff table
mag_cpl_to_cutoff_dict = {}
mag_cpl_threshold_list = []
for line in open(cutoff_table):
    if not line.startswith('MAG	PWY	Enzyme'):
        line_split = line.strip().split()
        mag_c = int(line_split[0])
        mag_e = int(line_split[1])
        mag_p = int(line_split[2])
        mag_cpl_to_cutoff_dict[mag_c] = [mag_e, mag_p]
        mag_cpl_threshold_list.append(mag_c)


mag_completeness_dict = {}
mag_to_pathway_cutoff_dict = {}
mag_to_key_enzyme_cutoff_dict = {}
for mag in open(mag_completeness):
    mag_split = mag.strip().split('\t')
    mag_id = mag_split[0]
    mag_cpl = float(mag_split[1])
    mag_completeness_dict[mag_id] = mag_cpl
    current_mag_pathway_cutoff, current_mag_key_enzyme_cutoff = get_pwy_ec_cutoff_by_mag_cpl(mag_cpl, mag_cpl_to_cutoff_dict, mag_cpl_threshold_list)
    mag_to_pathway_cutoff_dict[mag_id] = current_mag_pathway_cutoff
    mag_to_key_enzyme_cutoff_dict[mag_id] = current_mag_key_enzyme_cutoff


print(mag_completeness_dict)
print(mag_to_key_enzyme_cutoff_dict)
print(mag_to_pathway_cutoff_dict)




import os


db_file_raw =           '/Users/songweizhi/DB/MetaCyc/all_reactions_with_identified_enzyme.txt'
db_file_with_ec_tmp =   '/Users/songweizhi/DB/MetaCyc/all_reactions_with_ec_tmp.txt'
db_file_with_ec =       '/Users/songweizhi/DB/MetaCyc/all_reactions_with_ec.txt'

db_file_with_ec_handle = open(db_file_with_ec_tmp, 'w')
for each in open(db_file_raw):
    each_split = each.strip().split('\t')
    if len(each_split) > 1:
        if ',' in each_split[1]:
            enzyme_list = each_split[1].split(', ')
            for enzyme in enzyme_list:
                db_file_with_ec_handle.write('%s\t%s\n' % (enzyme, each_split[0]))
        else:
            db_file_with_ec_handle.write('%s\t%s\n' % (each_split[1], each_split[0]))
db_file_with_ec_handle.close()


os.system('cat %s | sort > %s' % (db_file_with_ec_tmp, db_file_with_ec))
os.system('rm %s' % db_file_with_ec_tmp)



import os
import glob
import pandas
import argparse
import numpy as np


ko00001_keg = '/Users/songweizhi/DB/KEGG_DB/ko00001.keg'


for each_line in open(ko00001_keg):
    if each_line[0] in ['A', 'B', 'C']:
        print(each_line.strip())


remaining_ABC = '''


'''



import sys
import argparse
from my_calculator_modules.get_sum import get_sum
from my_calculator_modules.get_product import get_product
from my_calculator_modules.get_square import get_square
from my_calculator_modules.get_cube import get_cube


def print_main_help():

    help_message = '''    
    .....::::: my_calculator :::::.....
               
    add       -> calculate sum 
    multiply  -> calculate product
    square    -> calculate square
    cube      -> calculate cube
    
    Usage: my_calculator <command> -h for command specific help
    '''

    print(help_message)


# initialize the options parser
parser = argparse.ArgumentParser()
subparsers = parser.add_subparsers(help="--", dest='subparser_name')


# arguments for add
add_parser = subparsers.add_parser('add',
                                   description='calculate sum of specified two numbers',
                                   epilog='Example: my_calculator add -a1 5 -a2 7')

add_parser.add_argument('-a1', required=True, type=int, help='input number 1')
add_parser.add_argument('-a2', required=True, type=int, help='input number 2')


# arguments for multiply
multiply_parser = subparsers.add_parser('multiply',
                                        description='calculate multiply',
                                        epilog='Example: my_calculator square -m1 5 -m2 6')

multiply_parser.add_argument('-m1', required=True, type=int, help='input number 1')
multiply_parser.add_argument('-m2', required=True, type=int, help='input number 2')


# arguments for square
square_parser = subparsers.add_parser('square',
                                      description='calculate square',
                                      epilog='Example: my_calculator square -s 6')

square_parser.add_argument('-s', required=True, type=int, help='input number')


# arguments for cube
cube_parser = subparsers.add_parser('cube',
                                    description='calculate cube',
                                    epilog='Example: my_calculator cube -c 5')

cube_parser.add_argument('-c', required=True, type=int, help='input number')


# get and check options
args = None
if (len(sys.argv) == 1) or (sys.argv[1] == '-h') or (sys.argv == '--help'):
    print_main_help()
    sys.exit(0)
else:
    args = parser.parse_args()
args_vars = vars(args)


if args_vars['subparser_name'] == 'add':
    get_sum(args_vars)

if args_vars['subparser_name'] == 'multiply':
    get_product(args_vars)

if args_vars['subparser_name'] == 'square':
    get_square(args_vars)

if args_vars['subparser_name'] == 'cube':
    get_cube(args_vars)




#!/usr/bin/python3
import os
import re
#import dendropy
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord
from Bio import Phylo

# NEEDS, hmmer3, mafft and fasttree
# Call with:    1) Path to folder containing all prokka output folders
#               2) Path to its temporary output folder
#               3) Path to the phylo.hmm file
#               4) Path to write the final alignment and tree


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def get_species_tree_alignment(tmp_folder, path_to_prokka, pwd_hmmsearch_exe, pwd_mafft_exe):

    # Tests for presence of the tmp folder and deletes it
    if os.path.exists(tmp_folder):
        os.system('rm -r ' + tmp_folder)
    os.mkdir(tmp_folder)

    # List all prokka dirs in the target folder
    prokka_files = [i for i in os.listdir(path_to_prokka) if os.path.isdir(path_to_prokka + '/' + i)]
    print('Detected %i input genomes' % len(prokka_files))

    # Running hmmsearch on each file
    print('Running hmmsearch...')
    for f in prokka_files:
        # call hmmsearch
        os.system('%s -o /dev/null --domtblout %s/%s_hmmout.tbl %s %s/%s/%s.faa' % (pwd_hmmsearch_exe, tmp_folder, f, path_to_hmm, path_to_prokka, f, f))

        # Reading the protein file in a dictionary
        proteinSequence = {}
        for seq_record in SeqIO.parse('%s/%s/%s.faa' % (path_to_prokka, f, f), 'fasta'):
            proteinSequence[seq_record.id] = str(seq_record.seq)

        # Reading the hmmersearch table/extracting the protein part found beu hmmsearch out of the protein/Writing each protein sequence that was extracted to a fasta file (one for each hmm in phylo.hmm
        proteinName = ''
        for line in open('%s/%s_hmmout.tbl' % (tmp_folder, f), 'r'):
            if line[0] != "#":
                line = re.sub('\s+', ' ', line) # ro remove redundant spaces
                splitLine = line.split(' ')
                if proteinName != splitLine[3]:
                    proteinName = splitLine[3]
                    file_out = open(tmp_folder + '/' + proteinName + '.fasta', 'a+')
                    x1 = int(splitLine[17]) - 1
                    x2 = int(splitLine[18])
                    export_dna_record(proteinSequence[splitLine[0]][x1: x2], f, '', file_out)
                    file_out.close()

    # Call mafft to align all single fasta files with hmms
    files = os.listdir(tmp_folder)
    fastaFiles = [i for i in files if i.endswith('.fasta')]
    print('Running mafft...')
    for f in fastaFiles:
        fastaFile1 = '%s/%s' % (tmp_folder, f)
        fastaFile2 = fastaFile1.replace('.fasta', '_aligned.fasta')
        os.system(pwd_mafft_exe + ' --quiet --maxiterate 1000 --globalpair ' + fastaFile1 + ' > ' + fastaFile2 + ' ; rm ' + fastaFile1)


def get_species_tree_newick(tmp_folder, each_subset, pwd_fasttree_exe):
    # concatenating the single alignments
    concatAlignment = {}
    for element in each_subset:
        concatAlignment[element] = ''

    # Reading all single alignment files and append them to the concatenated alignment
    files = os.listdir(tmp_folder)
    fastaFiles = [i for i in files if i.endswith('.fasta')]
    for f in fastaFiles:
        fastaFile = tmp_folder + '/' + f
        proteinSequence = {}
        alignmentLength = 0
        for seq_record_2 in SeqIO.parse(fastaFile, 'fasta'):
            proteinName = seq_record_2.id
            proteinSequence[proteinName] = str(seq_record_2.seq)
            alignmentLength = len(proteinSequence[proteinName])

        for element in each_subset:
            if element in proteinSequence.keys():
                concatAlignment[element] += proteinSequence[element]
            else:
                concatAlignment[element] += '-' * alignmentLength

    # writing alignment to file
    file_out = open('./alignment.fasta', 'w')
    # for element in prokka_files:
    for element in each_subset:
        file_out.write('>' + element + '\n' + concatAlignment[element] + '\n')
    file_out.close()

    # calling fasttree for tree calculation
    os.system('%s -quiet alignment.fasta > phylogenticTree.phy' % pwd_fasttree_exe)

    # Decomment the two following lines if tree is rooted but should be unrooted
    # phyloTree = dendropy.Tree.get(path='phylogenticTree.phy', schema='newick', rooting='force-unrooted')
    # dendropy.Tree.write_to_path(phyloTree, 'phylogenticTree_unrooted.phy', 'newick')
    # os.system('rm -r ' + tmp_folder)
    species_tree = Phylo.read('phylogenticTree.phy', 'newick')
    Phylo.draw_ascii(species_tree)


os.chdir('/Users/songweizhi/Desktop/hmmTree')
pwd_hmmsearch_exe = '/Users/songweizhi/Softwares/hmmer/hmmer-3.1b2-macosx-intel/binaries/hmmsearch'
pwd_fasttree_exe = '/Users/songweizhi/Softwares/FastTree/fasttree'
pwd_mafft_exe = 'mafft'
path_to_prokka = 'prokka'
path_to_hmm = 'phylo.hmm'
tmp_folder = 'hmm_tmp'
subset_list = [['AMS', 'AAM', 'AKV', 'AMAU', 'AMAC', 'BDS', 'BAD', 'BGC', 'BNM', 'BHS'],
               ['AMS', 'AAM', 'AKV', 'AMAU', 'AMAC'],
               ['BDS', 'BAD', 'BGC', 'BNM', 'BHS'],
               ['AKV', 'AMAU', 'BDS', 'BAD']]


get_species_tree_alignment(tmp_folder, path_to_prokka, pwd_hmmsearch_exe, pwd_mafft_exe)


for each_subset in subset_list:
    get_species_tree_newick(tmp_folder, each_subset, pwd_fasttree_exe)
